(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('http'), require('https'), require('url'), require('assert'), require('stream'), require('tty'), require('util'), require('zlib')) :
    typeof define === 'function' && define.amd ? define(['exports', 'http', 'https', 'url', 'assert', 'stream', 'tty', 'util', 'zlib'], factory) :
    (global = global || self, factory(global.mider = {}, global.http, global.https, global.url, global.assert, global.stream, global.tty, global.util, global.zlib));
}(this, function (exports, http, https, url, assert, stream, tty, util, zlib) { 'use strict';

    http = http && http.hasOwnProperty('default') ? http['default'] : http;
    https = https && https.hasOwnProperty('default') ? https['default'] : https;
    url = url && url.hasOwnProperty('default') ? url['default'] : url;
    assert = assert && assert.hasOwnProperty('default') ? assert['default'] : assert;
    stream = stream && stream.hasOwnProperty('default') ? stream['default'] : stream;
    tty = tty && tty.hasOwnProperty('default') ? tty['default'] : tty;
    util = util && util.hasOwnProperty('default') ? util['default'] : util;
    zlib = zlib && zlib.hasOwnProperty('default') ? zlib['default'] : zlib;

    /**
     * 类型检查
     * @param type
     * @returns {function(*=): boolean}
     */
    function isType(type) {
        return function (obj) {
            return {}.toString.call(obj) === "[object " + type + "]";
        }
    }

    /**
     * 类型检查 Undefined
     * @function
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isUndefined = isType("Undefined");
    /**
     * 类型检查 Object
     * @function
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isObject = isType("Object");
    /**
     * 类型检查 Number
     * @function
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isNumber = isType("Number");
    /**
     * 类型检查 Array
     * @function
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isArray = Array.isArray || isType("Array");
    /**
     * 类型检查 Function
     * @function
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isFunction = isType("Function");
    /**
     * 类型检查 String 验证 字符串字面量 和 new String()
     * @param {*} obj 检查目标
     * @global
     * @returns {boolean}
     * */
    let isString = function (obj) {
        return isType("String")(obj) || obj instanceof String;
    };

    //=========================================================

    /**
     * 对象复制
     * 其中的成员如果为对象{object}，则也进行复制操作
     * @param {Object|Array|Number|String} obj 目标对象
     * @returns {*} 返回复制的对象
     * @memberof $wd
     */
    function objClone(obj) {
        let newObj = {};
        if (isArray(obj)) {
            newObj = obj.map(e => objClone(e));
            return newObj;
        }
        if (isObject(obj)) {
            for (let an in obj) {
                if (!obj.hasOwnProperty(an)) continue;
                if (isArray(obj[an]))
                    newObj[an] = obj[an].slice();
                else
                    newObj[an] = isObject(obj[an]) ? objClone(obj[an]) : obj[an];
            }
            return newObj;
        }
        return obj;
    }

    /**
     * 提取object对应路径的属性
     * @param data {object} 提取目标
     * @param path {string}提取路径 'a.b.c'
     * @returns {*} 如果找对对应路径的属性则返回属性值，找不到则返回undefined
     * @memberof $wd
     * @example
     *      // 示例
     *      let obj = {
         *         a: {
         *             ao: {
         *                aoo:100
         *             }
         *         }
         *      };
     *      $wd.getObjAttr(obj, 'a.ao');            // => {aoo:100}
     *      $wd.getObjAttr(obj, 'a.ao.aoo');        // => 100
     *      $wd.getObjAttr(obj, 'a.an.aoo');        // => undefined
     *
     */
    function getObjAttr(data, path) {
        let result = data;
        if (!path) return result;
        try {
            path.split('.').forEach(function (name) {
                result = result[name];
            });
        } catch (e) {
            return;
        }
        return result;
    }

    /**
     * 设置object对应路径的属性
     * 如果中间路径不存在则自动创建
     * @param {object} data  设置目标
     * @param {string} path  设置路径 'a.b.c'
     * @param {*} value 设置的新值
     * @returns {*} 如果找对对应路径的属性则返回属性值
     * @memberof $wd
     * @example
     *
     *    $wd.setObjAttr({
         *       a: {a1:998},
         *       b: 2,
         *       c: {
         *           c1: 3
         *       }
         *    }, 'a.aa.aaa', 998);
     *
     *    // => { a:{aa:{aaa:998},a1:998},b:2...}
     */
    function setObjAttr(data, path, value) {
        let result = data;
        if (!path) return result;
        try {
            path.split('.').forEach(function (name, index) {
                if (index === path.split('.').length - 1) {
                    result[name] = value;
                    return
                }
                if (!isObject(result[name])) result[name] = {};
                result = result[name];
            });
        } catch (e) {
            return;
        }
        return data;
    }

    /**
     * 对象属性覆盖
     * @param {object} obj1 被覆盖对象 会被修改
     * @param {object} obj2 覆盖对象
     * @param {(string|function)=} process 处理每个覆盖属性的回调 可选参数 会使用函数返回值作为新值
     * @returns {object} 被覆盖处理后的对象
     * @memberof $wd
     * @example
     *
     *      // 示例
     *      let obj1 = {a:1,b:2},obj2={a:3,c:4};
     *      $wd.objCover(obj1,obj2);
     *      // => {a:3,b:2,c:4}
     *      // obj1 = {a:3,b:2,c:4}
     *
     */
    function objCover(obj1, obj2, process) {
        for (let an in obj2) {
            if (!obj2.hasOwnProperty(an) || (process === 'one-to-one' && !obj1.hasOwnProperty(an))) continue;
            obj1[an] = isFunction(process) ? process(obj2[an], an, obj1[an]) : obj2[an];
        }
        return obj1;
    }

    /**
     * 对象属性覆盖
     * @param {object} obj 清空的对象 会被修改
     * @param {Number} [degree=VeryBig] 清空的深度
     * @returns {object} 处理后的对象
     * @memberof $wd
     * @example
     *
     *      // 示例
     *      let obj1 = {a:1,b:2};
     *      $wd.objClear(obj1,obj2);
     *      // => {a:null,b:null}
     *      // obj1 = {a:null,b:null}
     *
     */
    function objClear(obj, degree) {
        let searchDegree = degree >= 0 ? degree : -1 >>> 1;
        if (searchDegree < 1) return obj;
        if (isObject(obj)) {
            Object.keys(obj).forEach(pn => {
                if (isObject(obj[pn])) {
                    objClear(obj[pn], degree - 1);
                } else if (isArray(obj[pn])) {
                    obj[pn].splice(0, obj[pn].length);
                } else {
                    obj[pn] = null;
                }
            });
        }
        return obj;
    }

    /**
     * 比较两个对象是否相等
     * 默认进行==比较 ""==false==[]
     * @param obj1 对象1
     * @param obj2 对象2
     * @param attr {(string|array)=} 对象属性   a,b,c ['a','b','c']
     * @returns {boolean}
     * @memberof $wd
     * @example
     *
     *      // 示例
     *      let obj1 = {a:1,b:2},obj2={a:1,c:2},obj3={a:2};
     *      $wd.objEqual(obj1,obj2);
     *      // => true
     *      $wd.objEqual(obj1,obj3);
     *      // => false
     *
     */
    function objEqual(obj1, obj2, attr) {
        let attrList = [];
        if (obj1 === obj2)
            return true;
        if (!isObject(obj1) || !isObject(obj2))
            return false;
        for (let an in obj2) {
            if (attr) {
                if (isString(attr)) attrList = attr.split(",");
                if (attrList.filter(function (e) {
                    return e === an;
                }).length <= 0) continue;
            }
            if (!obj2.hasOwnProperty(an)) continue;
            if (obj2[an] === obj1[an]) continue;
            return false;
        }
        return true;
    }

    /**
     * 对象属性提取
     * 现在主要用于构造提交的表单数据
     * @param {object} obj 被提取属性的目标对象 可以为对象的数组  [obj , ...]
     * @param {(string[]|string)} options 提取规则   数组 : [option , ...]  option: 'oldName=>newName' | 'oldName=newName' | 'oldName'   字符串 : 'oldName=>newName,oldName2=>newName2'
     * @param {(boolean)=} [except=false] 第二个配置项的属性列表是否为排除模式
     * @returns {object|Array.<object>} 使用提取属性构造的新对象或数组
     * @memberof $wd
     * @example
     *
     *      // 从对象中抽取数据
     *      let obj = {red:255,name:'peter'};
     *      $wd.objExtract(obj,["red=>colorRed","name=>lastName"]);     // => { colorRed:255,lastName:'peter' }
     *      $wd.objExtract(obj,"red=>colorRed,name"]);                  // => { colorRed:255,name:'peter' }
     *
     *      // 从对象数组中抽取数据
     *      let arr = [obj,obj];
     *      $wd.objExtract(obj,["red=>colorRed","lastName"]);           // => [{ colorRed:255,lastName:'peter' },{ colorRed:255,name:'peter' }]
     *
     *      // 自动生成路径
     *      $wd.objExtract({
         *             a: 1,
         *             b: { b1: 2 },
         *             c: 3
         *          },
     *          "a,b.b1=>bb1,c=>z.x.c.v.b.n.m"
     *      );
     *      // 结果如下 =>
     *      //      {
         *      //         a: 1,
         *      //         bb1: 2,
         *      //         z:{x:{c:{v:{b:{n:{m:3}}}}}}
         *      //      }
     *
     */
    function objExtract(obj, options, except) {
        // 提取 对应名称的属性
        let result = [], temp, originObj = objClone(obj), originOpt = options;
        isUndefined(options) && (originOpt = []);
        // 支持 使用逗号分隔的字符串生成配置数组 "oldName=>newName , oldName=>newName"
        isString(options) && ((temp = options) && (originOpt = options.split(',')));
        // 支持 从数组提取对象数组 obj [{a:1},{a:2}]  => [{b:1},{b:2}]
        !isArray(obj) && ((temp = originObj) && (originObj = []) && originObj.push(temp));
        originObj.forEach(function (unit) {
            // except pattern
            if (except) {
                temp = unit;
                originOpt.forEach(function (name) {
                    delete temp[name];
                });
            }
            // extract pattern
            if (!except) {
                temp = {};
                originOpt.forEach(function (name) {
                    let resolveReg = /([^=>,]+)=[>]?([^=>,]+)/;     // 支持中文
                    if (resolveReg.test(name)) {
                        // 支持 'oldName=>newName' 'oldName=newName'
                        let originName = resolveReg.exec(name)[1];
                        let replaceName = resolveReg.exec(name)[2];
                        !isUndefined(getObjAttr(unit, originName)) && (setObjAttr(temp, replaceName, getObjAttr(unit, originName)));
                    } else {
                        // 支持 'name'
                        !isUndefined(getObjAttr(unit, name)) && (setObjAttr(temp, name, getObjAttr(unit, name)));
                    }
                });
            }
            result.push(temp);
        });
        if (!isArray(obj)) return result[0];
        return result;
    }

    function objMix(options, coverOptions, degree) {
        let searchDegree = degree >= 0 ? degree : -1 >>> 1;
        if (searchDegree < 1) return false;
        if (isUndefined(options) || isUndefined(coverOptions)) {
            return false;
        }
        if (isArray(coverOptions)) {
            if (!isArray(options)) {
                options = [];
            }
            coverOptions.forEach((value, index) => {
                if (!objMix(options[index], coverOptions[index], searchDegree - 1)) {
                    options[index] = coverOptions[index];
                }
            });
            return true;
        }
        if (isObject(coverOptions)) {
            if (!isObject(options)) {
                options = {};
            }
            // keys & symbols
            Object.keys(coverOptions).concat(Object.getOwnPropertySymbols(coverOptions)).forEach(key => {
                if (!objMix(options[key], coverOptions[key], searchDegree - 1)) {
                    options[key] = coverOptions[key];
                }
            });
            return true;
        }
        return false;
    }

    //=========================================================

    /**
     * 解析URL 返回 解析数据对象
     * @param {string} url 需要解析的URL
     * @return {{source: *, query, origin: (*|string|string), params: {}, hashQuery, hashParams: {}, protocol: (string|XML|void|*), host, port, file: (*|string), hash: (string|XML|void|*), path: (string|XML|void|*), relative: (*|string), segments: Array, rebuild: rebuild}}
     * @memberof $wd
     */
    function parseURL(url) {
        let a = document.createElement('a');
        a.href = url;
        return {
            source: url,
            query: a.search,
            origin: a.origin || a.href.match(/\w+:\/\/[^\/\\]+/),
            params: function () {
                let ret = {},
                    seg = a.search.replace(/^\?/, '').split('&');
                if (seg == false) return {};
                seg.forEach(function (e, index, arr) {
                    arr[index] = {
                        name: e.split('=')[0],
                        value: decodeURI(e.split('=')[1])
                    };
                });
                seg.forEach(function (e) {
                    let name = e.name;
                    let obj = [];
                    if (typeof ret[name] === 'undefined') {
                        seg.forEach(function (e) {
                            if (e.name === name) obj.push(e.value);
                        });
                        if (obj.length === 1) ret[name] = obj[0];
                        else ret[name] = obj;
                    }
                });
                return ret;
            }(),
            hashQuery: function () {
                return (a.hash.match(/(?=\?).*/) && a.hash.match(/(?=\?).*/)[0]) || "";
            }(),
            hashParams: function () {
                if (!/.*\?/.test(a.hash)) return {};
                let ret = {},
                    seg = a.hash.replace(/.*\?/g, '').split('&');
                seg.forEach(function (e, index, arr) {
                    arr[index] = {
                        name: e.split('=')[0],
                        value: decodeURIComponent(e.split('=')[1])
                    };
                });
                seg.forEach(function (e) {
                    let name = e.name;
                    let obj = [];
                    if (typeof ret[name] === 'undefined') {
                        seg.forEach(function (e) {
                            if (e.name === name) obj.push(e.value);
                        });
                        if (obj.length === 1) ret[name] = obj[0];
                        else ret[name] = obj;
                    }
                });
                return ret;
            }(),
            protocol: a.protocol.replace(':', ''),
            host: a.hostname,
            port: a.port,
            file: (a.pathname.match(/\/([^\/?#]+)$/i) || ['', ''])[1],
            hash: a.hash.replace('#', ''),
            path: a.pathname.replace(/^([^\/])/, '/$1'),
            relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || ['', ''])[1],
            segments: a.pathname.replace(/^\//, '').split('/'),
            rebuild: function () {
                return buildURL({
                    url: this.url,
                    hash: this.hash,
                    origin: this.origin,
                    source: this.source,
                    path: this.path,
                    params: this.params,
                });
            }
        };
    }

    /**
     * 构建URL
     * @param {object} options
     * @param {string} [options.url=]       具体url
     * @param {string} [options.hash=]      hash
     * @param {string} [options.origin=]    域名
     * @param {string} [options.source=]    source
     * @param {string} [options.path=]      路径
     * @param {object} [options.params=]    参数
     * @returns {string}
     */
    function buildURL(options) {
        let urlInfo = parseURL(options.url || options.source);
        objCover(urlInfo, objExtract(options, "url", true));
        urlInfo.path = urlInfo.path.replace(/^(?=[^\/])/, '/');

        let paramsStr = "";
        Object.keys(urlInfo.params).forEach(key => {
            let value = urlInfo.params[key];
            if (value === null || value === undefined) return;
            if (!isArray(value)) value = [value];
            value.forEach(function (value) {
                if (isObject(value)) value = encodeURIComponent(JSON.stringify(value));
                paramsStr += (paramsStr ? "&" : "") + key + '=' + encodeURIComponent(value);
            });
        });
        return urlInfo.origin + urlInfo.path + (paramsStr ? "?" + paramsStr : "") + (urlInfo.hash ? "#" + urlInfo.hash : "");

    }

    /**
     * 数组操作兼容低版本浏览器
     */
    // https://tc39.github.io/ecma262/#sec-array.prototype.find
    if (!Array.prototype.find) {
        Object.defineProperty(Array.prototype, 'find', {
            value: function (predicate) {
                // 1. Let O be ? ToObject(this value).
                if (this == null) {
                    throw new TypeError('"this" is null or not defined');
                }

                var o = Object(this);

                // 2. Let len be ? ToLength(? Get(O, "length")).
                var len = o.length >>> 0;

                // 3. If IsCallable(predicate) is false, throw a TypeError exception.
                if (typeof predicate !== 'function') {
                    throw new TypeError('predicate must be a function');
                }

                // 4. If thisArg was supplied, let T be thisArg; else let T be undefined.
                var thisArg = arguments[1];

                // 5. Let k be 0.
                var k = 0;

                // 6. Repeat, while k < len
                while (k < len) {
                    // a. Let Pk be ! ToString(k).
                    // b. Let kValue be ? Get(O, Pk).
                    // c. Let testResult be ToBoolean(? Call(predicate, T, « kValue, k, O »)).
                    // d. If testResult is true, return kValue.
                    var kValue = o[k];
                    if (predicate.call(thisArg, kValue, k, o)) {
                        return kValue;
                    }
                    // e. Increase k by 1.
                    k++;
                }

                // 7. Return undefined.
                return undefined;
            },
            configurable: true,
            writable: true
        });
    }

    /**
     * 公共函数库
     * @author wdzxc
     */

    const $wd = {};

    //=========================================================
    //  兼容性处理
    //=========================================================
    // console
    console.log || (console.log = new Function());
    console.warn || (console.warn = console.log);
    console.error || (console.error = console.log);

    //=========================================================
    //  异步加载资源
    //=========================================================
    /**
     * 获取JS
     * @param {string} src JS路径
     * @param {function} callback 获取成功后回调
     * @returns {boolean} 返回true表示页面上没有这个JS开始进行加载，返回false表示已经存在这个JS文件不进行加载
     * @memberof $wd
     */
    function getScript(src, callback) {
        // 页面脚本
        if (src) {
            let curScripts = document.querySelectorAll('script');
            for (let i = 0; i < curScripts.length; i++) {
                if (curScripts[i].getAttribute('src') === src) {
                    callback('exist');
                    return false;
                }
            }
            let pageScript = document.createElement('script');
            pageScript.type = 'text/javascript';
            pageScript.src = src;
            document.head.appendChild(pageScript);
            pageScript.onload = function () {
                console.log("script load success [" + this.getAttribute('src') + "]");
                callback(true);
            };
            pageScript.onerror = function () {
                console.log("script load error [" + this.getAttribute('src') + "]");
                callback('error');
            };
        }
    }

    //=========================================================
    //  对外提供的功能
    //=========================================================


    $wd.objClone = objClone;
    $wd.getObjAttr = getObjAttr;

    $wd.parseURL = parseURL;
    $wd.buildURL = buildURL;

    $wd.objCover = objCover;
    $wd.objClear = objClear;
    $wd.objEqual = objEqual;
    $wd.objExtract = objExtract;
    $wd.objMix = objMix;
    $wd.setObjAttr = setObjAttr;

    $wd.getScript = getScript;

    // export
    $wd.$wd = $wd;
    $wd.isArray = isArray;
    $wd.isNumber = isNumber;
    $wd.isObject = isObject;
    $wd.isString = isString;
    $wd.isFunction = isFunction;
    $wd.isUndefined = isUndefined;

    /**
     * 文件上传组件
     * @author wdzxc
     */

    /**
     * 分析返回值  object | string => object
     * @param resContent
     */
    function parseRetData(resContent) {
        let retData;
        try {
            retData = JSON.parse(resContent);
        } catch (e) {
            retData = resContent;
        }
        return retData;
    }

    /**
     * 文件上传组件
     * @param {Object} config                        - 基础配置对象
     * @param {string} config.uploadUrl                - 文件上传接口URL
     * @param {string=} config.crossOriginSrc           - 跨域上传iframe代理URL
     * @param {string=} config.fileType                - 文件类型 需要小写字母 示例："txt,js,jpg,png,jpeg" | ['text','js']
     * @param {string=} config.fileSize                - 文件大小，单位kb 示例: "10-200" | "10~200" |"200"
     * @param {element} config.uploadElem                - 文件上传表单元素
     * @param {element=} config.imgElem                    - 图片文件预览元素
     * @param {function=} config.callback            - 成功回调
     * @param {function=} config.errCallback            - 失败回调
     * @param {function=} config.uploading            - 上传中回调
     * @param {function=} config.beforeSend            - 上传开始回调
     * @param {function=} config.onchange            - 文件选中、改变回调，可以在这个回调函数中写入 send方法
     * @returns {{send: send}}                            - 返回包含操作方法的对象
     * @constructor
     * @alias module:FileUploader
     * @example
     *   var fp = new FileUploader({
         *   	uploadUrl : "http://dove.zol.com.cn/saas.pin/ftp/fileUpload",
         *   	// 上传Form
         *   	uploadElem: fileUploadElem,
         *   	// 文件类型限制
         *   	fileType:"txt,js,jpg,png,jpeg",
         *   	// 文件大小限制
         *   	fileSize:"10-200",
         *   	// 跨域模式
         *   	crossOriginSrc:"http://dove.zol.com.cn/saas.pin/upload/fileSpy.html",
         *   	// 图片
         *   	imgElem : document.querySelector('img'),
         *   	//文件上传完成后回调函数
         *   	callback: function(resContent) {
         *   	    console.log(resContent);
         *   	},
         *   	errCallback: function(resContent) {
         *   	    console.log(resContent);
         *   	},
         *   	// 进度
         *   	uploading: function(percent) {
         *   	    console.log("Uploading:" + percent);
         *   	},
         *   	// onchange
         *   	onchange:function(){
         *   	    console.log("change");
         *   	}
         * });
     * // 发送文件
     * fp.send();
     *
     *
     *
     */
    let FileUploader = function (config) {
        let options = {
            // url
            uploadUrl: "",
            // formElem
            uploadElem: null,
            // success
            callback: null,
            errCallback: null,
            // uploading
            uploading: null,
            // beforeSend
            beforeSend: null,
            // img preview elem
            imgElem: null,
            // onchange
            onchange: null,
            // 文件类型
            fileType: '',
            // 文件大小 kb
            fileSize: null,
            // Authorization
            Authorization: false,
            // synergy {FileUploader}
            partner: null
        };
        // 参数合并
        for (let an in config) {
            if (!config.hasOwnProperty(an))continue;
            options[an] = config[an];
        }

        // 文件类型&大小检查
        function fileInspect(file) {
            if (!file) {
                options.errCallback && options.errCallback("没有选中任何文件");
                return false;
            }
            // 检查文件类型是否符合标准
            let typeMatch = true;
            if (options.fileType) {
                typeMatch = false;
                if (isString(options.fileType)) {
                    options.fileType = options.fileType.split(',');
                }
                if (isArray(options.fileType)) {
                    options.fileType.forEach(function (type) {
                        isString(type) && file.name.match(/\.[^.]*$/) && file.name.match(/\.[^.]*$/)[0].toLowerCase().indexOf(type) >= 0 && (typeMatch = true);
                    });
                }
            }
            if (!typeMatch) {
                options.errCallback && options.errCallback("文件类型错误！正确类型:" + options.fileType.join(','));
                return false;
            }
            if (isString(options.fileSize)) {
                let fileLimit = options.fileSize.split(/~|-|,/);
                let compareSize = file.size / 1024;
                if (fileLimit.length == 2) {
                    if (compareSize < fileLimit[0] || compareSize > fileLimit[1]) {
                        options.errCallback && options.errCallback("文件大小错误(" + parseInt(compareSize) + "kb)!尺寸范围：" + fileLimit.join('~') + ' kb');
                        return false;
                    }
                }
                else if (fileLimit.length == 1) {
                    if (compareSize > fileLimit[0]) {
                        options.errCallback && options.errCallback("文件大小错误(" + parseInt(compareSize) + "kb)！文件需要小于 " + fileLimit[0] + " kb");
                        return false;
                    }
                } else {
                    console.log("Error : 文件大小配置错误！");
                    return false;
                }
            }
            return true;
        }

        /*
         * XHR上传文件
         */
        let xhr = new XMLHttpRequest();
        options.uploadElem.addEventListener('change', function () {
            previewImage();
        });

        // 创建文件的URL
        let createObjectURL = function (blob) {
            return window[window["webkitURL"] ? 'webkitURL' : 'URL']['createObjectURL'](blob);
        };

        // 预览图片
        function previewImage(imageFile) {

            // 预览参数图片 或 表单图片
            let file = imageFile || options.uploadElem.files[0];
            if (!fileInspect(file)) {
                options.uploadElem.value = "";
                return;
            }

            // IE10
            if (options.imgElem) {
                options.imgElem.src = createObjectURL(file);
                options.imgElem.style.opacity = 0.2;
            }

            // 触发change事件
            options.onchange && options.onchange();
        }

        return {
            extractFiles: function (targetFiles) {
                let files = targetFiles || options.uploadElem.files;
                let file = files[0];
                if (!file) {
                    options.errCallback && options.errCallback("没有选中任何文件");
                    return;
                }
                // 处理多选的文件
                if (options.partner && files.length > 1) {
                    options.partner.synergy([].slice.call(files, 1));
                }
                return file;
            },
            send: function (targetFiles) {
                let file = this.extractFiles(targetFiles);
                // 构造表单数据
                let formData = new FormData();
                isFunction(options.beforeSend) && options.beforeSend(file);
                formData.append("files", file);
                // 上传状态
                xhr.onreadystatechange = function () {
                    if (xhr.readyState == 4) {
                        if (xhr.status == 200) {
                            options.callback && options.callback(parseRetData(xhr.responseText));
                        } else {
                            options.errCallback && options.errCallback("ServerError:" + xhr.responseText);
                        }
                    }
                };
                /**
                 * 上传进度
                 * @param {{loaded,total}} evt
                 */
                xhr.upload.onprogress = function (evt) {
                    let loaded = evt.loaded;
                    let total = evt.total;
                    let percent = Math.floor(100 * loaded / total);
                    console.log(percent);
                    options.imgElem.style.opacity = percent / 100;
                    options.uploading && options.uploading(percent);
                };
                xhr.open("post", options.uploadUrl);
                xhr.send(formData);
            },
            // 传递协同 额外文件
            synergy: function (remainFiles) {
                if (options.partner && remainFiles) {
                    setTimeout(function () {
                        options.partner.send(remainFiles);
                    }, 300);
                }
                previewImage(remainFiles[0]);
            },
            options: options
        }
    };
    FileUploader.prototype = {};

    var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

    function createCommonjsModule(fn, module) {
    	return module = { exports: {} }, fn(module, module.exports), module.exports;
    }

    function getCjsExportFromNamespace (n) {
    	return n && n['default'] || n;
    }

    var sha1 = createCommonjsModule(function (module) {
    /**
     * component-sha1
     */

    /**
     * Configurable variables
     */

    (function($) {
        var b64pad  = "="; /* base-64 pad character. "=" for strict RFC compliance   */
        var chrsz   = 8;   /* bits per input character. 8 - ASCII; 16 - Unicode      */

        /**
         * Calculate the SHA-1 of an array of big-endian words, and a bit length
         */

        var core_sha1 = function(x, len) {
            /* append padding */
            x[len >> 5] |= 0x80 << (24 - len % 32);
            x[((len + 64 >> 9) << 4) + 15] = len;

            var w = Array(80);
            var a =  1732584193;
            var b = -271733879;
            var c = -1732584194;
            var d =  271733878;
            var e = -1009589776;

            var i;
            var j;

            for(i = 0; i < x.length; i += 16) {
                var olda = a;
                var oldb = b;
                var oldc = c;
                var oldd = d;
                var olde = e;

                for(j = 0; j < 80; j += 1) {
                    if(j < 16) {
                        w[j] = x[i + j];
                    } else {
                        w[j] = rol(w[j-3] ^ w[j-8] ^ w[j-14] ^ w[j-16], 1);
                    }

                    var t = safe_add(safe_add(rol(a, 5), sha1_ft(j, b, c, d)), safe_add(safe_add(e, w[j]), sha1_kt(j)));
                    e = d;
                    d = c;
                    c = rol(b, 30);
                    b = a;
                    a = t;
                }

                a = safe_add(a, olda);
                b = safe_add(b, oldb);
                c = safe_add(c, oldc);
                d = safe_add(d, oldd);
                e = safe_add(e, olde);
            }

            return Array(a, b, c, d, e);
        };

        /**
         * Perform the appropriate triplet combination function
         * for the current iteration
         */

        var sha1_ft = function(t, b, c, d) {
            if(t < 20) return (b & c) | ((~b) & d);
            if(t < 40) return b ^ c ^ d;
            if(t < 60) return (b & c) | (b & d) | (c & d);

            return b ^ c ^ d;
        };

        /**
         * Determine the appropriate additive constant for the current iteration
         */

        var sha1_kt = function(t) {
            return (t < 20) ?  1518500249 : (t < 40) ?  1859775393 :
                (t < 60) ? -1894007588 : -899497514;
        };

        /**
         * Calculate the HMAC-SHA1 of a key and some data
         */

        var core_hmac_sha1 = function(key, data) {
            var bkey = str2binb(key);
            if (bkey.length > 16) {
                bkey = core_sha1(bkey, key.length * chrsz);
            }
            var ipad = Array(16);
            var opad = Array(16);
            var i;

            for (i = 0; i < 16; i += 1) {
                ipad[i] = bkey[i] ^ 0x36363636;
                opad[i] = bkey[i] ^ 0x5C5C5C5C;
            }

            var hash = core_sha1(ipad.concat(str2binb(data)), 512 + data.length * chrsz);

            return core_sha1(opad.concat(hash), 512 + 160);
        };

        /**
         * Add integers, wrapping at 2^32.
         * This uses 16-bit operations internally to work around bugs in some JS interpreters.
         */

        var safe_add = function(x, y) {
            var lsw = (x & 0xFFFF) + (y & 0xFFFF);
            var msw = (x >> 16) + (y >> 16) + (lsw >> 16);

            return (msw << 16) | (lsw & 0xFFFF);
        };

        /**
         * Bitwise rotate a 32-bit number to the left.
         */

        var rol = function(num, cnt){
            return (num << cnt) | (num >>> (32 - cnt));
        };

        /**
         * Convert an 8-bit or 16-bit string to an array of big-endian words
         * In 8-bit function, characters >255 have their hi-byte silently ignored.
         */

        var str2binb = function(str) {
            var bin = [];
            var mask = (1 << chrsz) - 1;
            var i;

            for(i = 0; i < str.length * chrsz; i += chrsz) {
                bin[i>>5] |= (str.charCodeAt(i / chrsz) & mask) << (32 - chrsz - i%32);
            }

            return bin;
        };

        /**
         * Convert an array of big-endian words to a string
         */

        var binb2str = function(bin) {
            var str = "";
            var mask = (1 << chrsz) - 1;
            var i;

            for(i = 0; i < bin.length * 32; i += chrsz) {
                str += String.fromCharCode((bin[i>>5] >>> (32 - chrsz - i%32)) & mask);
            }

            return str;
        };

        /*
         * Convert an array of big-endian words to a hex string.
         */
        var binb2hex = function(binarray) {
            var hex_tab = "0123456789abcdef";
            var str = "";
            var i;

            for(i = 0; i < binarray.length * 4; i += 1) {
                str += hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8+4)) & 0xF) +
                    hex_tab.charAt((binarray[i>>2] >> ((3 - i%4)*8  )) & 0xF);
            }

            return str;
        };

        /**
         * Convert an array of big-endian words to a base-64 string
         */

        var binb2b64 = function(binarray) {
            var tab = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
            var str = "";
            var i;
            var j;

            for(i = 0; i < binarray.length * 4; i += 3) {
                var triplet = (((binarray[i   >> 2] >> 8 * (3 -  i   %4)) & 0xFF) << 16)
                    | (((binarray[i+1 >> 2] >> 8 * (3 - (i+1)%4)) & 0xFF) << 8 )
                    |  ((binarray[i+2 >> 2] >> 8 * (3 - (i+2)%4)) & 0xFF);
                for(j = 0; j < 4; j += 1) {
                    if (i * 8 + j * 6 > binarray.length * 32) {
                        str += b64pad;
                    } else {
                        str += tab.charAt((triplet >> 6*(3-j)) & 0x3F);
                    }
                }
            }

            return str;
        };

        /**
         * export
         */

        var exports = {};

        /**
         * hex
         * Hashes a string using SHA1 and returns a hex representation of it.
         *
         * @param {String} s
         * @returns {String}
         * @api public
         */

        exports.hex = function(s) {
            return binb2hex(core_sha1(str2binb(s), s.length * chrsz));
        };

        /**
         * base64
         * Hashes a string using SHA1 and returns a base64 representation of it.
         *
         * @param {String} s
         * @returns {String}
         * @api public
         */

        exports.base64 = function(s) {
            return binb2b64(core_sha1(str2binb(s),s.length * chrsz));
        };

        /**
         * str
         * Hashes a string using SHA1 and returns a string representation of it.
         *
         * @param {String} s
         * @returns {String}
         * @api public
         */

        exports.str = function(s) {
            return binb2str(core_sha1(str2binb(s),s.length * chrsz));
        };

        /**
         * hex_hmac
         * Calculates the HMAC-SHA1 of a key and some data,
         * returning a hex representation of it.
         *
         * @param {String} key
         * @param {String} data
         * @returns {String}
         * @api public
         */

        exports.hex_hmac = function(key, data) {
            return binb2hex(core_hmac_sha1(key, data));
        };

        /**
         * base64_hmac
         * Calculates the HMAC-SHA1 of a key and some data,
         * returning a base64 representation of it.
         *
         * @param {String} key
         * @param {String} data
         * @returns {String}
         * @api public
         */

        exports.base64_hmac = function(key, data) {
            return binb2b64(core_hmac_sha1(key, data));
        };

        /**
         * str_hmac
         * Calculates the HMAC-SHA1 of a key and some data,
         * returning a string representation of it.
         *
         * @param {String} key
         * @param {String} data
         * @returns {String}
         * @api public
         */

        exports.str_hmac = function(key, data) {
            return binb2str(core_hmac_sha1(key, data));
        };

        if (module.exports) {
            module.exports = exports;
        } else {
            $.sha1 = exports;
        }
    }(commonjsGlobal));
    });

    /**
     * jsonp
     */

    // default ajax params
    const defaultOptions = {
        url: '',
        data: null,
        dataType: 'json',
        jsonp: 'callback',
        success: new Function,
        error: new Function,
    };
    //
    let jsonpCounter = 100;

    function ajaxJsonp(config) {
        // init options
        let options = objCover(objClone(defaultOptions), config);
        // create receive func
        let jsonpCallbackName = 'jsonp_cb' + jsonpCounter++;
        let called = false;
        window[jsonpCallbackName] = function (data) {
            called = true;
            options.success(data);
        };
        // script tag
        let pageScript = document.createElement('script');
        pageScript.type = 'text/javascript';
        let params = objClone(options.data);
        params[options.jsonp] = jsonpCallbackName;
        params["_"] = +new Date();
        pageScript.src = buildURL({
            url: options.url,
            params: params
        });
        document.head.appendChild(pageScript);
        pageScript.onload = function () {
            if (called === false) {
                options.error({
                    message: 'Jsonp response illegal',
                });
            }
            delete window[jsonpCallbackName];
            document.head.removeChild(pageScript);
        };
        pageScript.onerror = function () {
            options.error({
                message: 'Jsonp request error',
            });
            document.head.removeChild(pageScript);
        };
    }

    var bind = function bind(fn, thisArg) {
      return function wrap() {
        var args = new Array(arguments.length);
        for (var i = 0; i < args.length; i++) {
          args[i] = arguments[i];
        }
        return fn.apply(thisArg, args);
      };
    };

    /*!
     * Determine if an object is a Buffer
     *
     * @author   Feross Aboukhadijeh <https://feross.org>
     * @license  MIT
     */

    var isBuffer = function isBuffer (obj) {
      return obj != null && obj.constructor != null &&
        typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
    };

    /*global toString:true*/

    // utils is a library of generic helper functions non-specific to axios

    var toString = Object.prototype.toString;

    /**
     * Determine if a value is an Array
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is an Array, otherwise false
     */
    function isArray$1(val) {
      return toString.call(val) === '[object Array]';
    }

    /**
     * Determine if a value is an ArrayBuffer
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is an ArrayBuffer, otherwise false
     */
    function isArrayBuffer(val) {
      return toString.call(val) === '[object ArrayBuffer]';
    }

    /**
     * Determine if a value is a FormData
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is an FormData, otherwise false
     */
    function isFormData(val) {
      return (typeof FormData !== 'undefined') && (val instanceof FormData);
    }

    /**
     * Determine if a value is a view on an ArrayBuffer
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a view on an ArrayBuffer, otherwise false
     */
    function isArrayBufferView(val) {
      var result;
      if ((typeof ArrayBuffer !== 'undefined') && (ArrayBuffer.isView)) {
        result = ArrayBuffer.isView(val);
      } else {
        result = (val) && (val.buffer) && (val.buffer instanceof ArrayBuffer);
      }
      return result;
    }

    /**
     * Determine if a value is a String
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a String, otherwise false
     */
    function isString$1(val) {
      return typeof val === 'string';
    }

    /**
     * Determine if a value is a Number
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a Number, otherwise false
     */
    function isNumber$1(val) {
      return typeof val === 'number';
    }

    /**
     * Determine if a value is undefined
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if the value is undefined, otherwise false
     */
    function isUndefined$1(val) {
      return typeof val === 'undefined';
    }

    /**
     * Determine if a value is an Object
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is an Object, otherwise false
     */
    function isObject$1(val) {
      return val !== null && typeof val === 'object';
    }

    /**
     * Determine if a value is a Date
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a Date, otherwise false
     */
    function isDate(val) {
      return toString.call(val) === '[object Date]';
    }

    /**
     * Determine if a value is a File
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a File, otherwise false
     */
    function isFile(val) {
      return toString.call(val) === '[object File]';
    }

    /**
     * Determine if a value is a Blob
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a Blob, otherwise false
     */
    function isBlob(val) {
      return toString.call(val) === '[object Blob]';
    }

    /**
     * Determine if a value is a Function
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a Function, otherwise false
     */
    function isFunction$1(val) {
      return toString.call(val) === '[object Function]';
    }

    /**
     * Determine if a value is a Stream
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a Stream, otherwise false
     */
    function isStream(val) {
      return isObject$1(val) && isFunction$1(val.pipe);
    }

    /**
     * Determine if a value is a URLSearchParams object
     *
     * @param {Object} val The value to test
     * @returns {boolean} True if value is a URLSearchParams object, otherwise false
     */
    function isURLSearchParams(val) {
      return typeof URLSearchParams !== 'undefined' && val instanceof URLSearchParams;
    }

    /**
     * Trim excess whitespace off the beginning and end of a string
     *
     * @param {String} str The String to trim
     * @returns {String} The String freed of excess whitespace
     */
    function trim(str) {
      return str.replace(/^\s*/, '').replace(/\s*$/, '');
    }

    /**
     * Determine if we're running in a standard browser environment
     *
     * This allows axios to run in a web worker, and react-native.
     * Both environments support XMLHttpRequest, but not fully standard globals.
     *
     * web workers:
     *  typeof window -> undefined
     *  typeof document -> undefined
     *
     * react-native:
     *  navigator.product -> 'ReactNative'
     */
    function isStandardBrowserEnv() {
      if (typeof navigator !== 'undefined' && navigator.product === 'ReactNative') {
        return false;
      }
      return (
        typeof window !== 'undefined' &&
        typeof document !== 'undefined'
      );
    }

    /**
     * Iterate over an Array or an Object invoking a function for each item.
     *
     * If `obj` is an Array callback will be called passing
     * the value, index, and complete array for each item.
     *
     * If 'obj' is an Object callback will be called passing
     * the value, key, and complete object for each property.
     *
     * @param {Object|Array} obj The object to iterate
     * @param {Function} fn The callback to invoke for each item
     */
    function forEach(obj, fn) {
      // Don't bother if no value provided
      if (obj === null || typeof obj === 'undefined') {
        return;
      }

      // Force an array if not already something iterable
      if (typeof obj !== 'object') {
        /*eslint no-param-reassign:0*/
        obj = [obj];
      }

      if (isArray$1(obj)) {
        // Iterate over array values
        for (var i = 0, l = obj.length; i < l; i++) {
          fn.call(null, obj[i], i, obj);
        }
      } else {
        // Iterate over object keys
        for (var key in obj) {
          if (Object.prototype.hasOwnProperty.call(obj, key)) {
            fn.call(null, obj[key], key, obj);
          }
        }
      }
    }

    /**
     * Accepts varargs expecting each argument to be an object, then
     * immutably merges the properties of each object and returns result.
     *
     * When multiple objects contain the same key the later object in
     * the arguments list will take precedence.
     *
     * Example:
     *
     * ```js
     * var result = merge({foo: 123}, {foo: 456});
     * console.log(result.foo); // outputs 456
     * ```
     *
     * @param {Object} obj1 Object to merge
     * @returns {Object} Result of all merge properties
     */
    function merge(/* obj1, obj2, obj3, ... */) {
      var result = {};
      function assignValue(val, key) {
        if (typeof result[key] === 'object' && typeof val === 'object') {
          result[key] = merge(result[key], val);
        } else {
          result[key] = val;
        }
      }

      for (var i = 0, l = arguments.length; i < l; i++) {
        forEach(arguments[i], assignValue);
      }
      return result;
    }

    /**
     * Extends object a by mutably adding to it the properties of object b.
     *
     * @param {Object} a The object to be extended
     * @param {Object} b The object to copy properties from
     * @param {Object} thisArg The object to bind function to
     * @return {Object} The resulting value of object a
     */
    function extend(a, b, thisArg) {
      forEach(b, function assignValue(val, key) {
        if (thisArg && typeof val === 'function') {
          a[key] = bind(val, thisArg);
        } else {
          a[key] = val;
        }
      });
      return a;
    }

    var utils = {
      isArray: isArray$1,
      isArrayBuffer: isArrayBuffer,
      isBuffer: isBuffer,
      isFormData: isFormData,
      isArrayBufferView: isArrayBufferView,
      isString: isString$1,
      isNumber: isNumber$1,
      isObject: isObject$1,
      isUndefined: isUndefined$1,
      isDate: isDate,
      isFile: isFile,
      isBlob: isBlob,
      isFunction: isFunction$1,
      isStream: isStream,
      isURLSearchParams: isURLSearchParams,
      isStandardBrowserEnv: isStandardBrowserEnv,
      forEach: forEach,
      merge: merge,
      extend: extend,
      trim: trim
    };

    var normalizeHeaderName = function normalizeHeaderName(headers, normalizedName) {
      utils.forEach(headers, function processHeader(value, name) {
        if (name !== normalizedName && name.toUpperCase() === normalizedName.toUpperCase()) {
          headers[normalizedName] = value;
          delete headers[name];
        }
      });
    };

    /**
     * Update an Error with the specified config, error code, and response.
     *
     * @param {Error} error The error to update.
     * @param {Object} config The config.
     * @param {string} [code] The error code (for example, 'ECONNABORTED').
     * @param {Object} [request] The request.
     * @param {Object} [response] The response.
     * @returns {Error} The error.
     */
    var enhanceError = function enhanceError(error, config, code, request, response) {
      error.config = config;
      if (code) {
        error.code = code;
      }
      error.request = request;
      error.response = response;
      return error;
    };

    /**
     * Create an Error with the specified message, config, error code, request and response.
     *
     * @param {string} message The error message.
     * @param {Object} config The config.
     * @param {string} [code] The error code (for example, 'ECONNABORTED').
     * @param {Object} [request] The request.
     * @param {Object} [response] The response.
     * @returns {Error} The created error.
     */
    var createError = function createError(message, config, code, request, response) {
      var error = new Error(message);
      return enhanceError(error, config, code, request, response);
    };

    /**
     * Resolve or reject a Promise based on response status.
     *
     * @param {Function} resolve A function that resolves the promise.
     * @param {Function} reject A function that rejects the promise.
     * @param {object} response The response.
     */
    var settle = function settle(resolve, reject, response) {
      var validateStatus = response.config.validateStatus;
      // Note: status is not exposed by XDomainRequest
      if (!response.status || !validateStatus || validateStatus(response.status)) {
        resolve(response);
      } else {
        reject(createError(
          'Request failed with status code ' + response.status,
          response.config,
          null,
          response.request,
          response
        ));
      }
    };

    function encode(val) {
      return encodeURIComponent(val).
        replace(/%40/gi, '@').
        replace(/%3A/gi, ':').
        replace(/%24/g, '$').
        replace(/%2C/gi, ',').
        replace(/%20/g, '+').
        replace(/%5B/gi, '[').
        replace(/%5D/gi, ']');
    }

    /**
     * Build a URL by appending params to the end
     *
     * @param {string} url The base of the url (e.g., http://www.google.com)
     * @param {object} [params] The params to be appended
     * @returns {string} The formatted url
     */
    var buildURL$1 = function buildURL(url, params, paramsSerializer) {
      /*eslint no-param-reassign:0*/
      if (!params) {
        return url;
      }

      var serializedParams;
      if (paramsSerializer) {
        serializedParams = paramsSerializer(params);
      } else if (utils.isURLSearchParams(params)) {
        serializedParams = params.toString();
      } else {
        var parts = [];

        utils.forEach(params, function serialize(val, key) {
          if (val === null || typeof val === 'undefined') {
            return;
          }

          if (utils.isArray(val)) {
            key = key + '[]';
          } else {
            val = [val];
          }

          utils.forEach(val, function parseValue(v) {
            if (utils.isDate(v)) {
              v = v.toISOString();
            } else if (utils.isObject(v)) {
              v = JSON.stringify(v);
            }
            parts.push(encode(key) + '=' + encode(v));
          });
        });

        serializedParams = parts.join('&');
      }

      if (serializedParams) {
        url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
      }

      return url;
    };

    // Headers whose duplicates are ignored by node
    // c.f. https://nodejs.org/api/http.html#http_message_headers
    var ignoreDuplicateOf = [
      'age', 'authorization', 'content-length', 'content-type', 'etag',
      'expires', 'from', 'host', 'if-modified-since', 'if-unmodified-since',
      'last-modified', 'location', 'max-forwards', 'proxy-authorization',
      'referer', 'retry-after', 'user-agent'
    ];

    /**
     * Parse headers into an object
     *
     * ```
     * Date: Wed, 27 Aug 2014 08:58:49 GMT
     * Content-Type: application/json
     * Connection: keep-alive
     * Transfer-Encoding: chunked
     * ```
     *
     * @param {String} headers Headers needing to be parsed
     * @returns {Object} Headers parsed into an object
     */
    var parseHeaders = function parseHeaders(headers) {
      var parsed = {};
      var key;
      var val;
      var i;

      if (!headers) { return parsed; }

      utils.forEach(headers.split('\n'), function parser(line) {
        i = line.indexOf(':');
        key = utils.trim(line.substr(0, i)).toLowerCase();
        val = utils.trim(line.substr(i + 1));

        if (key) {
          if (parsed[key] && ignoreDuplicateOf.indexOf(key) >= 0) {
            return;
          }
          if (key === 'set-cookie') {
            parsed[key] = (parsed[key] ? parsed[key] : []).concat([val]);
          } else {
            parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
          }
        }
      });

      return parsed;
    };

    var isURLSameOrigin = (
      utils.isStandardBrowserEnv() ?

      // Standard browser envs have full support of the APIs needed to test
      // whether the request URL is of the same origin as current location.
      (function standardBrowserEnv() {
        var msie = /(msie|trident)/i.test(navigator.userAgent);
        var urlParsingNode = document.createElement('a');
        var originURL;

        /**
        * Parse a URL to discover it's components
        *
        * @param {String} url The URL to be parsed
        * @returns {Object}
        */
        function resolveURL(url) {
          var href = url;

          if (msie) {
            // IE needs attribute set twice to normalize properties
            urlParsingNode.setAttribute('href', href);
            href = urlParsingNode.href;
          }

          urlParsingNode.setAttribute('href', href);

          // urlParsingNode provides the UrlUtils interface - http://url.spec.whatwg.org/#urlutils
          return {
            href: urlParsingNode.href,
            protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
            host: urlParsingNode.host,
            search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
            hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
            hostname: urlParsingNode.hostname,
            port: urlParsingNode.port,
            pathname: (urlParsingNode.pathname.charAt(0) === '/') ?
                      urlParsingNode.pathname :
                      '/' + urlParsingNode.pathname
          };
        }

        originURL = resolveURL(window.location.href);

        /**
        * Determine if a URL shares the same origin as the current location
        *
        * @param {String} requestURL The URL to test
        * @returns {boolean} True if URL shares the same origin, otherwise false
        */
        return function isURLSameOrigin(requestURL) {
          var parsed = (utils.isString(requestURL)) ? resolveURL(requestURL) : requestURL;
          return (parsed.protocol === originURL.protocol &&
                parsed.host === originURL.host);
        };
      })() :

      // Non standard browser envs (web workers, react-native) lack needed support.
      (function nonStandardBrowserEnv() {
        return function isURLSameOrigin() {
          return true;
        };
      })()
    );

    var cookies = (
      utils.isStandardBrowserEnv() ?

      // Standard browser envs support document.cookie
      (function standardBrowserEnv() {
        return {
          write: function write(name, value, expires, path, domain, secure) {
            var cookie = [];
            cookie.push(name + '=' + encodeURIComponent(value));

            if (utils.isNumber(expires)) {
              cookie.push('expires=' + new Date(expires).toGMTString());
            }

            if (utils.isString(path)) {
              cookie.push('path=' + path);
            }

            if (utils.isString(domain)) {
              cookie.push('domain=' + domain);
            }

            if (secure === true) {
              cookie.push('secure');
            }

            document.cookie = cookie.join('; ');
          },

          read: function read(name) {
            var match = document.cookie.match(new RegExp('(^|;\\s*)(' + name + ')=([^;]*)'));
            return (match ? decodeURIComponent(match[3]) : null);
          },

          remove: function remove(name) {
            this.write(name, '', Date.now() - 86400000);
          }
        };
      })() :

      // Non standard browser env (web workers, react-native) lack needed support.
      (function nonStandardBrowserEnv() {
        return {
          write: function write() {},
          read: function read() { return null; },
          remove: function remove() {}
        };
      })()
    );

    var xhr = function xhrAdapter(config) {
      return new Promise(function dispatchXhrRequest(resolve, reject) {
        var requestData = config.data;
        var requestHeaders = config.headers;

        if (utils.isFormData(requestData)) {
          delete requestHeaders['Content-Type']; // Let the browser set it
        }

        var request = new XMLHttpRequest();

        // HTTP basic authentication
        if (config.auth) {
          var username = config.auth.username || '';
          var password = config.auth.password || '';
          requestHeaders.Authorization = 'Basic ' + btoa(username + ':' + password);
        }

        request.open(config.method.toUpperCase(), buildURL$1(config.url, config.params, config.paramsSerializer), true);

        // Set the request timeout in MS
        request.timeout = config.timeout;

        // Listen for ready state
        request.onreadystatechange = function handleLoad() {
          if (!request || request.readyState !== 4) {
            return;
          }

          // The request errored out and we didn't get a response, this will be
          // handled by onerror instead
          // With one exception: request that using file: protocol, most browsers
          // will return status as 0 even though it's a successful request
          if (request.status === 0 && !(request.responseURL && request.responseURL.indexOf('file:') === 0)) {
            return;
          }

          // Prepare the response
          var responseHeaders = 'getAllResponseHeaders' in request ? parseHeaders(request.getAllResponseHeaders()) : null;
          var responseData = !config.responseType || config.responseType === 'text' ? request.responseText : request.response;
          var response = {
            data: responseData,
            status: request.status,
            statusText: request.statusText,
            headers: responseHeaders,
            config: config,
            request: request
          };

          settle(resolve, reject, response);

          // Clean up request
          request = null;
        };

        // Handle low level network errors
        request.onerror = function handleError() {
          // Real errors are hidden from us by the browser
          // onerror should only fire if it's a network error
          reject(createError('Network Error', config, null, request));

          // Clean up request
          request = null;
        };

        // Handle timeout
        request.ontimeout = function handleTimeout() {
          reject(createError('timeout of ' + config.timeout + 'ms exceeded', config, 'ECONNABORTED',
            request));

          // Clean up request
          request = null;
        };

        // Add xsrf header
        // This is only done if running in a standard browser environment.
        // Specifically not if we're in a web worker, or react-native.
        if (utils.isStandardBrowserEnv()) {
          var cookies$1 = cookies;

          // Add xsrf header
          var xsrfValue = (config.withCredentials || isURLSameOrigin(config.url)) && config.xsrfCookieName ?
              cookies$1.read(config.xsrfCookieName) :
              undefined;

          if (xsrfValue) {
            requestHeaders[config.xsrfHeaderName] = xsrfValue;
          }
        }

        // Add headers to the request
        if ('setRequestHeader' in request) {
          utils.forEach(requestHeaders, function setRequestHeader(val, key) {
            if (typeof requestData === 'undefined' && key.toLowerCase() === 'content-type') {
              // Remove Content-Type if data is undefined
              delete requestHeaders[key];
            } else {
              // Otherwise add header to the request
              request.setRequestHeader(key, val);
            }
          });
        }

        // Add withCredentials to request if needed
        if (config.withCredentials) {
          request.withCredentials = true;
        }

        // Add responseType to request if needed
        if (config.responseType) {
          try {
            request.responseType = config.responseType;
          } catch (e) {
            // Expected DOMException thrown by browsers not compatible XMLHttpRequest Level 2.
            // But, this can be suppressed for 'json' type as it can be parsed by default 'transformResponse' function.
            if (config.responseType !== 'json') {
              throw e;
            }
          }
        }

        // Handle progress if needed
        if (typeof config.onDownloadProgress === 'function') {
          request.addEventListener('progress', config.onDownloadProgress);
        }

        // Not all browsers support upload events
        if (typeof config.onUploadProgress === 'function' && request.upload) {
          request.upload.addEventListener('progress', config.onUploadProgress);
        }

        if (config.cancelToken) {
          // Handle cancellation
          config.cancelToken.promise.then(function onCanceled(cancel) {
            if (!request) {
              return;
            }

            request.abort();
            reject(cancel);
            // Clean up request
            request = null;
          });
        }

        if (requestData === undefined) {
          requestData = null;
        }

        // Send the request
        request.send(requestData);
      });
    };

    /**
     * Helpers.
     */

    var s = 1000;
    var m = s * 60;
    var h = m * 60;
    var d = h * 24;
    var y = d * 365.25;

    /**
     * Parse or format the given `val`.
     *
     * Options:
     *
     *  - `long` verbose formatting [false]
     *
     * @param {String|Number} val
     * @param {Object} [options]
     * @throws {Error} throw an error if val is not a non-empty string or a number
     * @return {String|Number}
     * @api public
     */

    var ms = function(val, options) {
      options = options || {};
      var type = typeof val;
      if (type === 'string' && val.length > 0) {
        return parse(val);
      } else if (type === 'number' && isNaN(val) === false) {
        return options.long ? fmtLong(val) : fmtShort(val);
      }
      throw new Error(
        'val is not a non-empty string or a valid number. val=' +
          JSON.stringify(val)
      );
    };

    /**
     * Parse the given `str` and return milliseconds.
     *
     * @param {String} str
     * @return {Number}
     * @api private
     */

    function parse(str) {
      str = String(str);
      if (str.length > 100) {
        return;
      }
      var match = /^((?:\d+)?\.?\d+) *(milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|years?|yrs?|y)?$/i.exec(
        str
      );
      if (!match) {
        return;
      }
      var n = parseFloat(match[1]);
      var type = (match[2] || 'ms').toLowerCase();
      switch (type) {
        case 'years':
        case 'year':
        case 'yrs':
        case 'yr':
        case 'y':
          return n * y;
        case 'days':
        case 'day':
        case 'd':
          return n * d;
        case 'hours':
        case 'hour':
        case 'hrs':
        case 'hr':
        case 'h':
          return n * h;
        case 'minutes':
        case 'minute':
        case 'mins':
        case 'min':
        case 'm':
          return n * m;
        case 'seconds':
        case 'second':
        case 'secs':
        case 'sec':
        case 's':
          return n * s;
        case 'milliseconds':
        case 'millisecond':
        case 'msecs':
        case 'msec':
        case 'ms':
          return n;
        default:
          return undefined;
      }
    }

    /**
     * Short format for `ms`.
     *
     * @param {Number} ms
     * @return {String}
     * @api private
     */

    function fmtShort(ms) {
      if (ms >= d) {
        return Math.round(ms / d) + 'd';
      }
      if (ms >= h) {
        return Math.round(ms / h) + 'h';
      }
      if (ms >= m) {
        return Math.round(ms / m) + 'm';
      }
      if (ms >= s) {
        return Math.round(ms / s) + 's';
      }
      return ms + 'ms';
    }

    /**
     * Long format for `ms`.
     *
     * @param {Number} ms
     * @return {String}
     * @api private
     */

    function fmtLong(ms) {
      return plural(ms, d, 'day') ||
        plural(ms, h, 'hour') ||
        plural(ms, m, 'minute') ||
        plural(ms, s, 'second') ||
        ms + ' ms';
    }

    /**
     * Pluralization helper.
     */

    function plural(ms, n, name) {
      if (ms < n) {
        return;
      }
      if (ms < n * 1.5) {
        return Math.floor(ms / n) + ' ' + name;
      }
      return Math.ceil(ms / n) + ' ' + name + 's';
    }

    var debug = createCommonjsModule(function (module, exports) {
    /**
     * This is the common logic for both the Node.js and web browser
     * implementations of `debug()`.
     *
     * Expose `debug()` as the module.
     */

    exports = module.exports = createDebug.debug = createDebug['default'] = createDebug;
    exports.coerce = coerce;
    exports.disable = disable;
    exports.enable = enable;
    exports.enabled = enabled;
    exports.humanize = ms;

    /**
     * Active `debug` instances.
     */
    exports.instances = [];

    /**
     * The currently active debug mode names, and names to skip.
     */

    exports.names = [];
    exports.skips = [];

    /**
     * Map of special "%n" handling functions, for the debug "format" argument.
     *
     * Valid key names are a single, lower or upper-case letter, i.e. "n" and "N".
     */

    exports.formatters = {};

    /**
     * Select a color.
     * @param {String} namespace
     * @return {Number}
     * @api private
     */

    function selectColor(namespace) {
      var hash = 0, i;

      for (i in namespace) {
        hash  = ((hash << 5) - hash) + namespace.charCodeAt(i);
        hash |= 0; // Convert to 32bit integer
      }

      return exports.colors[Math.abs(hash) % exports.colors.length];
    }

    /**
     * Create a debugger with the given `namespace`.
     *
     * @param {String} namespace
     * @return {Function}
     * @api public
     */

    function createDebug(namespace) {

      var prevTime;

      function debug() {
        // disabled?
        if (!debug.enabled) return;

        var self = debug;

        // set `diff` timestamp
        var curr = +new Date();
        var ms = curr - (prevTime || curr);
        self.diff = ms;
        self.prev = prevTime;
        self.curr = curr;
        prevTime = curr;

        // turn the `arguments` into a proper Array
        var args = new Array(arguments.length);
        for (var i = 0; i < args.length; i++) {
          args[i] = arguments[i];
        }

        args[0] = exports.coerce(args[0]);

        if ('string' !== typeof args[0]) {
          // anything else let's inspect with %O
          args.unshift('%O');
        }

        // apply any `formatters` transformations
        var index = 0;
        args[0] = args[0].replace(/%([a-zA-Z%])/g, function(match, format) {
          // if we encounter an escaped % then don't increase the array index
          if (match === '%%') return match;
          index++;
          var formatter = exports.formatters[format];
          if ('function' === typeof formatter) {
            var val = args[index];
            match = formatter.call(self, val);

            // now we need to remove `args[index]` since it's inlined in the `format`
            args.splice(index, 1);
            index--;
          }
          return match;
        });

        // apply env-specific formatting (colors, etc.)
        exports.formatArgs.call(self, args);

        var logFn = debug.log || exports.log || console.log.bind(console);
        logFn.apply(self, args);
      }

      debug.namespace = namespace;
      debug.enabled = exports.enabled(namespace);
      debug.useColors = exports.useColors();
      debug.color = selectColor(namespace);
      debug.destroy = destroy;

      // env-specific initialization logic for debug instances
      if ('function' === typeof exports.init) {
        exports.init(debug);
      }

      exports.instances.push(debug);

      return debug;
    }

    function destroy () {
      var index = exports.instances.indexOf(this);
      if (index !== -1) {
        exports.instances.splice(index, 1);
        return true;
      } else {
        return false;
      }
    }

    /**
     * Enables a debug mode by namespaces. This can include modes
     * separated by a colon and wildcards.
     *
     * @param {String} namespaces
     * @api public
     */

    function enable(namespaces) {
      exports.save(namespaces);

      exports.names = [];
      exports.skips = [];

      var i;
      var split = (typeof namespaces === 'string' ? namespaces : '').split(/[\s,]+/);
      var len = split.length;

      for (i = 0; i < len; i++) {
        if (!split[i]) continue; // ignore empty strings
        namespaces = split[i].replace(/\*/g, '.*?');
        if (namespaces[0] === '-') {
          exports.skips.push(new RegExp('^' + namespaces.substr(1) + '$'));
        } else {
          exports.names.push(new RegExp('^' + namespaces + '$'));
        }
      }

      for (i = 0; i < exports.instances.length; i++) {
        var instance = exports.instances[i];
        instance.enabled = exports.enabled(instance.namespace);
      }
    }

    /**
     * Disable debug output.
     *
     * @api public
     */

    function disable() {
      exports.enable('');
    }

    /**
     * Returns true if the given mode name is enabled, false otherwise.
     *
     * @param {String} name
     * @return {Boolean}
     * @api public
     */

    function enabled(name) {
      if (name[name.length - 1] === '*') {
        return true;
      }
      var i, len;
      for (i = 0, len = exports.skips.length; i < len; i++) {
        if (exports.skips[i].test(name)) {
          return false;
        }
      }
      for (i = 0, len = exports.names.length; i < len; i++) {
        if (exports.names[i].test(name)) {
          return true;
        }
      }
      return false;
    }

    /**
     * Coerce `val`.
     *
     * @param {Mixed} val
     * @return {Mixed}
     * @api private
     */

    function coerce(val) {
      if (val instanceof Error) return val.stack || val.message;
      return val;
    }
    });
    var debug_1 = debug.coerce;
    var debug_2 = debug.disable;
    var debug_3 = debug.enable;
    var debug_4 = debug.enabled;
    var debug_5 = debug.humanize;
    var debug_6 = debug.instances;
    var debug_7 = debug.names;
    var debug_8 = debug.skips;
    var debug_9 = debug.formatters;

    var browser = createCommonjsModule(function (module, exports) {
    /**
     * This is the web browser implementation of `debug()`.
     *
     * Expose `debug()` as the module.
     */

    exports = module.exports = debug;
    exports.log = log;
    exports.formatArgs = formatArgs;
    exports.save = save;
    exports.load = load;
    exports.useColors = useColors;
    exports.storage = 'undefined' != typeof chrome
                   && 'undefined' != typeof chrome.storage
                      ? chrome.storage.local
                      : localstorage();

    /**
     * Colors.
     */

    exports.colors = [
      '#0000CC', '#0000FF', '#0033CC', '#0033FF', '#0066CC', '#0066FF', '#0099CC',
      '#0099FF', '#00CC00', '#00CC33', '#00CC66', '#00CC99', '#00CCCC', '#00CCFF',
      '#3300CC', '#3300FF', '#3333CC', '#3333FF', '#3366CC', '#3366FF', '#3399CC',
      '#3399FF', '#33CC00', '#33CC33', '#33CC66', '#33CC99', '#33CCCC', '#33CCFF',
      '#6600CC', '#6600FF', '#6633CC', '#6633FF', '#66CC00', '#66CC33', '#9900CC',
      '#9900FF', '#9933CC', '#9933FF', '#99CC00', '#99CC33', '#CC0000', '#CC0033',
      '#CC0066', '#CC0099', '#CC00CC', '#CC00FF', '#CC3300', '#CC3333', '#CC3366',
      '#CC3399', '#CC33CC', '#CC33FF', '#CC6600', '#CC6633', '#CC9900', '#CC9933',
      '#CCCC00', '#CCCC33', '#FF0000', '#FF0033', '#FF0066', '#FF0099', '#FF00CC',
      '#FF00FF', '#FF3300', '#FF3333', '#FF3366', '#FF3399', '#FF33CC', '#FF33FF',
      '#FF6600', '#FF6633', '#FF9900', '#FF9933', '#FFCC00', '#FFCC33'
    ];

    /**
     * Currently only WebKit-based Web Inspectors, Firefox >= v31,
     * and the Firebug extension (any Firefox version) are known
     * to support "%c" CSS customizations.
     *
     * TODO: add a `localStorage` variable to explicitly enable/disable colors
     */

    function useColors() {
      // NB: In an Electron preload script, document will be defined but not fully
      // initialized. Since we know we're in Chrome, we'll just detect this case
      // explicitly
      if (typeof window !== 'undefined' && window.process && window.process.type === 'renderer') {
        return true;
      }

      // Internet Explorer and Edge do not support colors.
      if (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/(edge|trident)\/(\d+)/)) {
        return false;
      }

      // is webkit? http://stackoverflow.com/a/16459606/376773
      // document is undefined in react-native: https://github.com/facebook/react-native/pull/1632
      return (typeof document !== 'undefined' && document.documentElement && document.documentElement.style && document.documentElement.style.WebkitAppearance) ||
        // is firebug? http://stackoverflow.com/a/398120/376773
        (typeof window !== 'undefined' && window.console && (window.console.firebug || (window.console.exception && window.console.table))) ||
        // is firefox >= v31?
        // https://developer.mozilla.org/en-US/docs/Tools/Web_Console#Styling_messages
        (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31) ||
        // double check webkit in userAgent just in case we are in a worker
        (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/applewebkit\/(\d+)/));
    }

    /**
     * Map %j to `JSON.stringify()`, since no Web Inspectors do that by default.
     */

    exports.formatters.j = function(v) {
      try {
        return JSON.stringify(v);
      } catch (err) {
        return '[UnexpectedJSONParseError]: ' + err.message;
      }
    };


    /**
     * Colorize log arguments if enabled.
     *
     * @api public
     */

    function formatArgs(args) {
      var useColors = this.useColors;

      args[0] = (useColors ? '%c' : '')
        + this.namespace
        + (useColors ? ' %c' : ' ')
        + args[0]
        + (useColors ? '%c ' : ' ')
        + '+' + exports.humanize(this.diff);

      if (!useColors) return;

      var c = 'color: ' + this.color;
      args.splice(1, 0, c, 'color: inherit');

      // the final "%c" is somewhat tricky, because there could be other
      // arguments passed either before or after the %c, so we need to
      // figure out the correct index to insert the CSS into
      var index = 0;
      var lastC = 0;
      args[0].replace(/%[a-zA-Z%]/g, function(match) {
        if ('%%' === match) return;
        index++;
        if ('%c' === match) {
          // we only are interested in the *last* %c
          // (the user may have provided their own)
          lastC = index;
        }
      });

      args.splice(lastC, 0, c);
    }

    /**
     * Invokes `console.log()` when available.
     * No-op when `console.log` is not a "function".
     *
     * @api public
     */

    function log() {
      // this hackery is required for IE8/9, where
      // the `console.log` function doesn't have 'apply'
      return 'object' === typeof console
        && console.log
        && Function.prototype.apply.call(console.log, console, arguments);
    }

    /**
     * Save `namespaces`.
     *
     * @param {String} namespaces
     * @api private
     */

    function save(namespaces) {
      try {
        if (null == namespaces) {
          exports.storage.removeItem('debug');
        } else {
          exports.storage.debug = namespaces;
        }
      } catch(e) {}
    }

    /**
     * Load `namespaces`.
     *
     * @return {String} returns the previously persisted debug modes
     * @api private
     */

    function load() {
      var r;
      try {
        r = exports.storage.debug;
      } catch(e) {}

      // If debug isn't set in LS, and we're in Electron, try to load $DEBUG
      if (!r && typeof process !== 'undefined' && 'env' in process) {
        r = process.env.DEBUG;
      }

      return r;
    }

    /**
     * Enable namespaces listed in `localStorage.debug` initially.
     */

    exports.enable(load());

    /**
     * Localstorage attempts to return the localstorage.
     *
     * This is necessary because safari throws
     * when a user disables cookies/localstorage
     * and you attempt to access it.
     *
     * @return {LocalStorage}
     * @api private
     */

    function localstorage() {
      try {
        return window.localStorage;
      } catch (e) {}
    }
    });
    var browser_1 = browser.log;
    var browser_2 = browser.formatArgs;
    var browser_3 = browser.save;
    var browser_4 = browser.load;
    var browser_5 = browser.useColors;
    var browser_6 = browser.storage;
    var browser_7 = browser.colors;

    var argv = process.argv;

    var terminator = argv.indexOf('--');
    var hasFlag = function (flag) {
    	flag = '--' + flag;
    	var pos = argv.indexOf(flag);
    	return pos !== -1 && (terminator !== -1 ? pos < terminator : true);
    };

    var supportsColor = (function () {
    	if ('FORCE_COLOR' in process.env) {
    		return true;
    	}

    	if (hasFlag('no-color') ||
    		hasFlag('no-colors') ||
    		hasFlag('color=false')) {
    		return false;
    	}

    	if (hasFlag('color') ||
    		hasFlag('colors') ||
    		hasFlag('color=true') ||
    		hasFlag('color=always')) {
    		return true;
    	}

    	if (process.stdout && !process.stdout.isTTY) {
    		return false;
    	}

    	if (process.platform === 'win32') {
    		return true;
    	}

    	if ('COLORTERM' in process.env) {
    		return true;
    	}

    	if (process.env.TERM === 'dumb') {
    		return false;
    	}

    	if (/^screen|^xterm|^vt100|color|ansi|cygwin|linux/i.test(process.env.TERM)) {
    		return true;
    	}

    	return false;
    })();

    var node = createCommonjsModule(function (module, exports) {
    /**
     * Module dependencies.
     */




    /**
     * This is the Node.js implementation of `debug()`.
     *
     * Expose `debug()` as the module.
     */

    exports = module.exports = debug;
    exports.init = init;
    exports.log = log;
    exports.formatArgs = formatArgs;
    exports.save = save;
    exports.load = load;
    exports.useColors = useColors;

    /**
     * Colors.
     */

    exports.colors = [ 6, 2, 3, 4, 5, 1 ];

    try {
      var supportsColor$1 = supportsColor;
      if (supportsColor$1 && supportsColor$1.level >= 2) {
        exports.colors = [
          20, 21, 26, 27, 32, 33, 38, 39, 40, 41, 42, 43, 44, 45, 56, 57, 62, 63, 68,
          69, 74, 75, 76, 77, 78, 79, 80, 81, 92, 93, 98, 99, 112, 113, 128, 129, 134,
          135, 148, 149, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171,
          172, 173, 178, 179, 184, 185, 196, 197, 198, 199, 200, 201, 202, 203, 204,
          205, 206, 207, 208, 209, 214, 215, 220, 221
        ];
      }
    } catch (err) {
      // swallow - we only care if `supports-color` is available; it doesn't have to be.
    }

    /**
     * Build up the default `inspectOpts` object from the environment variables.
     *
     *   $ DEBUG_COLORS=no DEBUG_DEPTH=10 DEBUG_SHOW_HIDDEN=enabled node script.js
     */

    exports.inspectOpts = Object.keys(process.env).filter(function (key) {
      return /^debug_/i.test(key);
    }).reduce(function (obj, key) {
      // camel-case
      var prop = key
        .substring(6)
        .toLowerCase()
        .replace(/_([a-z])/g, function (_, k) { return k.toUpperCase() });

      // coerce string value into JS value
      var val = process.env[key];
      if (/^(yes|on|true|enabled)$/i.test(val)) val = true;
      else if (/^(no|off|false|disabled)$/i.test(val)) val = false;
      else if (val === 'null') val = null;
      else val = Number(val);

      obj[prop] = val;
      return obj;
    }, {});

    /**
     * Is stdout a TTY? Colored output is enabled when `true`.
     */

    function useColors() {
      return 'colors' in exports.inspectOpts
        ? Boolean(exports.inspectOpts.colors)
        : tty.isatty(process.stderr.fd);
    }

    /**
     * Map %o to `util.inspect()`, all on a single line.
     */

    exports.formatters.o = function(v) {
      this.inspectOpts.colors = this.useColors;
      return util.inspect(v, this.inspectOpts)
        .split('\n').map(function(str) {
          return str.trim()
        }).join(' ');
    };

    /**
     * Map %o to `util.inspect()`, allowing multiple lines if needed.
     */

    exports.formatters.O = function(v) {
      this.inspectOpts.colors = this.useColors;
      return util.inspect(v, this.inspectOpts);
    };

    /**
     * Adds ANSI color escape codes if enabled.
     *
     * @api public
     */

    function formatArgs(args) {
      var name = this.namespace;
      var useColors = this.useColors;

      if (useColors) {
        var c = this.color;
        var colorCode = '\u001b[3' + (c < 8 ? c : '8;5;' + c);
        var prefix = '  ' + colorCode + ';1m' + name + ' ' + '\u001b[0m';

        args[0] = prefix + args[0].split('\n').join('\n' + prefix);
        args.push(colorCode + 'm+' + exports.humanize(this.diff) + '\u001b[0m');
      } else {
        args[0] = getDate() + name + ' ' + args[0];
      }
    }

    function getDate() {
      if (exports.inspectOpts.hideDate) {
        return '';
      } else {
        return new Date().toISOString() + ' ';
      }
    }

    /**
     * Invokes `util.format()` with the specified arguments and writes to stderr.
     */

    function log() {
      return process.stderr.write(util.format.apply(util, arguments) + '\n');
    }

    /**
     * Save `namespaces`.
     *
     * @param {String} namespaces
     * @api private
     */

    function save(namespaces) {
      if (null == namespaces) {
        // If you set a process.env field to null or undefined, it gets cast to the
        // string 'null' or 'undefined'. Just delete instead.
        delete process.env.DEBUG;
      } else {
        process.env.DEBUG = namespaces;
      }
    }

    /**
     * Load `namespaces`.
     *
     * @return {String} returns the previously persisted debug modes
     * @api private
     */

    function load() {
      return process.env.DEBUG;
    }

    /**
     * Init logic for `debug` instances.
     *
     * Create a new `inspectOpts` object in case `useColors` is set
     * differently for a particular `debug` instance.
     */

    function init (debug) {
      debug.inspectOpts = {};

      var keys = Object.keys(exports.inspectOpts);
      for (var i = 0; i < keys.length; i++) {
        debug.inspectOpts[keys[i]] = exports.inspectOpts[keys[i]];
      }
    }

    /**
     * Enable namespaces listed in `process.env.DEBUG` initially.
     */

    exports.enable(load());
    });
    var node_1 = node.init;
    var node_2 = node.log;
    var node_3 = node.formatArgs;
    var node_4 = node.save;
    var node_5 = node.load;
    var node_6 = node.useColors;
    var node_7 = node.colors;
    var node_8 = node.inspectOpts;

    var src = createCommonjsModule(function (module) {
    /**
     * Detect Electron renderer process, which is node, but we should
     * treat as a browser.
     */

    if (typeof process === 'undefined' || process.type === 'renderer') {
      module.exports = browser;
    } else {
      module.exports = node;
    }
    });

    var Writable = stream.Writable;
    var debug$1 = src("follow-redirects");

    // RFC7231§4.2.1: Of the request methods defined by this specification,
    // the GET, HEAD, OPTIONS, and TRACE methods are defined to be safe.
    var SAFE_METHODS = { GET: true, HEAD: true, OPTIONS: true, TRACE: true };

    // Create handlers that pass events from native requests
    var eventHandlers = Object.create(null);
    ["abort", "aborted", "error", "socket", "timeout"].forEach(function (event) {
      eventHandlers[event] = function (arg) {
        this._redirectable.emit(event, arg);
      };
    });

    // An HTTP(S) request that can be redirected
    function RedirectableRequest(options, responseCallback) {
      // Initialize the request
      Writable.call(this);
      options.headers = options.headers || {};
      this._options = options;
      this._redirectCount = 0;
      this._redirects = [];
      this._requestBodyLength = 0;
      this._requestBodyBuffers = [];

      // Since http.request treats host as an alias of hostname,
      // but the url module interprets host as hostname plus port,
      // eliminate the host property to avoid confusion.
      if (options.host) {
        // Use hostname if set, because it has precedence
        if (!options.hostname) {
          options.hostname = options.host;
        }
        delete options.host;
      }

      // Attach a callback if passed
      if (responseCallback) {
        this.on("response", responseCallback);
      }

      // React to responses of native requests
      var self = this;
      this._onNativeResponse = function (response) {
        self._processResponse(response);
      };

      // Complete the URL object when necessary
      if (!options.pathname && options.path) {
        var searchPos = options.path.indexOf("?");
        if (searchPos < 0) {
          options.pathname = options.path;
        }
        else {
          options.pathname = options.path.substring(0, searchPos);
          options.search = options.path.substring(searchPos);
        }
      }

      // Perform the first request
      this._performRequest();
    }
    RedirectableRequest.prototype = Object.create(Writable.prototype);

    // Writes buffered data to the current native request
    RedirectableRequest.prototype.write = function (data, encoding, callback) {
      // Validate input and shift parameters if necessary
      if (!(typeof data === "string" || typeof data === "object" && ("length" in data))) {
        throw new Error("data should be a string, Buffer or Uint8Array");
      }
      if (typeof encoding === "function") {
        callback = encoding;
        encoding = null;
      }

      // Ignore empty buffers, since writing them doesn't invoke the callback
      // https://github.com/nodejs/node/issues/22066
      if (data.length === 0) {
        if (callback) {
          callback();
        }
        return;
      }
      // Only write when we don't exceed the maximum body length
      if (this._requestBodyLength + data.length <= this._options.maxBodyLength) {
        this._requestBodyLength += data.length;
        this._requestBodyBuffers.push({ data: data, encoding: encoding });
        this._currentRequest.write(data, encoding, callback);
      }
      // Error when we exceed the maximum body length
      else {
        this.emit("error", new Error("Request body larger than maxBodyLength limit"));
        this.abort();
      }
    };

    // Ends the current native request
    RedirectableRequest.prototype.end = function (data, encoding, callback) {
      // Shift parameters if necessary
      if (typeof data === "function") {
        callback = data;
        data = encoding = null;
      }
      else if (typeof encoding === "function") {
        callback = encoding;
        encoding = null;
      }

      // Write data and end
      var currentRequest = this._currentRequest;
      this.write(data || "", encoding, function () {
        currentRequest.end(null, null, callback);
      });
    };

    // Sets a header value on the current native request
    RedirectableRequest.prototype.setHeader = function (name, value) {
      this._options.headers[name] = value;
      this._currentRequest.setHeader(name, value);
    };

    // Clears a header value on the current native request
    RedirectableRequest.prototype.removeHeader = function (name) {
      delete this._options.headers[name];
      this._currentRequest.removeHeader(name);
    };

    // Proxy all other public ClientRequest methods
    [
      "abort", "flushHeaders", "getHeader",
      "setNoDelay", "setSocketKeepAlive", "setTimeout",
    ].forEach(function (method) {
      RedirectableRequest.prototype[method] = function (a, b) {
        return this._currentRequest[method](a, b);
      };
    });

    // Proxy all public ClientRequest properties
    ["aborted", "connection", "socket"].forEach(function (property) {
      Object.defineProperty(RedirectableRequest.prototype, property, {
        get: function () { return this._currentRequest[property]; },
      });
    });

    // Executes the next native request (initial or redirect)
    RedirectableRequest.prototype._performRequest = function () {
      // Load the native protocol
      var protocol = this._options.protocol;
      var nativeProtocol = this._options.nativeProtocols[protocol];
      if (!nativeProtocol) {
        this.emit("error", new Error("Unsupported protocol " + protocol));
        return;
      }

      // If specified, use the agent corresponding to the protocol
      // (HTTP and HTTPS use different types of agents)
      if (this._options.agents) {
        var scheme = protocol.substr(0, protocol.length - 1);
        this._options.agent = this._options.agents[scheme];
      }

      // Create the native request
      var request = this._currentRequest =
            nativeProtocol.request(this._options, this._onNativeResponse);
      this._currentUrl = url.format(this._options);

      // Set up event handlers
      request._redirectable = this;
      for (var event in eventHandlers) {
        /* istanbul ignore else */
        if (event) {
          request.on(event, eventHandlers[event]);
        }
      }

      // End a redirected request
      // (The first request must be ended explicitly with RedirectableRequest#end)
      if (this._isRedirect) {
        // Write the request entity and end.
        var i = 0;
        var buffers = this._requestBodyBuffers;
        (function writeNext() {
          if (i < buffers.length) {
            var buffer = buffers[i++];
            request.write(buffer.data, buffer.encoding, writeNext);
          }
          else {
            request.end();
          }
        }());
      }
    };

    // Processes a response from the current native request
    RedirectableRequest.prototype._processResponse = function (response) {
      // Store the redirected response
      if (this._options.trackRedirects) {
        this._redirects.push({
          url: this._currentUrl,
          headers: response.headers,
          statusCode: response.statusCode,
        });
      }

      // RFC7231§6.4: The 3xx (Redirection) class of status code indicates
      // that further action needs to be taken by the user agent in order to
      // fulfill the request. If a Location header field is provided,
      // the user agent MAY automatically redirect its request to the URI
      // referenced by the Location field value,
      // even if the specific status code is not understood.
      var location = response.headers.location;
      if (location && this._options.followRedirects !== false &&
          response.statusCode >= 300 && response.statusCode < 400) {
        // RFC7231§6.4: A client SHOULD detect and intervene
        // in cyclical redirections (i.e., "infinite" redirection loops).
        if (++this._redirectCount > this._options.maxRedirects) {
          this.emit("error", new Error("Max redirects exceeded."));
          return;
        }

        // RFC7231§6.4: Automatic redirection needs to done with
        // care for methods not known to be safe […],
        // since the user might not wish to redirect an unsafe request.
        // RFC7231§6.4.7: The 307 (Temporary Redirect) status code indicates
        // that the target resource resides temporarily under a different URI
        // and the user agent MUST NOT change the request method
        // if it performs an automatic redirection to that URI.
        var header;
        var headers = this._options.headers;
        if (response.statusCode !== 307 && !(this._options.method in SAFE_METHODS)) {
          this._options.method = "GET";
          // Drop a possible entity and headers related to it
          this._requestBodyBuffers = [];
          for (header in headers) {
            if (/^content-/i.test(header)) {
              delete headers[header];
            }
          }
        }

        // Drop the Host header, as the redirect might lead to a different host
        if (!this._isRedirect) {
          for (header in headers) {
            if (/^host$/i.test(header)) {
              delete headers[header];
            }
          }
        }

        // Perform the redirected request
        var redirectUrl = url.resolve(this._currentUrl, location);
        debug$1("redirecting to", redirectUrl);
        Object.assign(this._options, url.parse(redirectUrl));
        this._isRedirect = true;
        this._performRequest();

        // Discard the remainder of the response to avoid waiting for data
        response.destroy();
      }
      else {
        // The response is not a redirect; return it as-is
        response.responseUrl = this._currentUrl;
        response.redirects = this._redirects;
        this.emit("response", response);

        // Clean up
        this._requestBodyBuffers = [];
      }
    };

    // Wraps the key/value object of protocols with redirect functionality
    function wrap(protocols) {
      // Default settings
      var exports = {
        maxRedirects: 21,
        maxBodyLength: 10 * 1024 * 1024,
      };

      // Wrap each protocol
      var nativeProtocols = {};
      Object.keys(protocols).forEach(function (scheme) {
        var protocol = scheme + ":";
        var nativeProtocol = nativeProtocols[protocol] = protocols[scheme];
        var wrappedProtocol = exports[scheme] = Object.create(nativeProtocol);

        // Executes a request, following redirects
        wrappedProtocol.request = function (options, callback) {
          if (typeof options === "string") {
            options = url.parse(options);
            options.maxRedirects = exports.maxRedirects;
          }
          else {
            options = Object.assign({
              protocol: protocol,
              maxRedirects: exports.maxRedirects,
              maxBodyLength: exports.maxBodyLength,
            }, options);
          }
          options.nativeProtocols = nativeProtocols;
          assert.equal(options.protocol, protocol, "protocol mismatch");
          debug$1("options", options);
          return new RedirectableRequest(options, callback);
        };

        // Executes a GET request, following redirects
        wrappedProtocol.get = function (options, callback) {
          var request = wrappedProtocol.request(options, callback);
          request.end();
          return request;
        };
      });
      return exports;
    }

    // Exports
    var followRedirects = wrap({ http: http, https: https });
    var wrap_1 = wrap;
    followRedirects.wrap = wrap_1;

    var _from = "axios@^0.18.0";
    var _id = "axios@0.18.1";
    var _inBundle = false;
    var _integrity = "sha1-/z8N4ue10YDnV62YAA8Qgbh7zqM=";
    var _location = "/axios";
    var _phantomChildren = {
    };
    var _requested = {
    	type: "range",
    	registry: true,
    	raw: "axios@^0.18.0",
    	name: "axios",
    	escapedName: "axios",
    	rawSpec: "^0.18.0",
    	saveSpec: null,
    	fetchSpec: "^0.18.0"
    };
    var _requiredBy = [
    	"/"
    ];
    var _resolved = "https://registry.npm.taobao.org/axios/download/axios-0.18.1.tgz?cache=0&other_urls=https%3A%2F%2Fregistry.npm.taobao.org%2Faxios%2Fdownload%2Faxios-0.18.1.tgz";
    var _shasum = "ff3f0de2e7b5d180e757ad98000f1081b87bcea3";
    var _spec = "axios@^0.18.0";
    var _where = "C:\\Workspace\\iyunxiao\\mider";
    var author = {
    	name: "Matt Zabriskie"
    };
    var browser$1 = {
    	"./lib/adapters/http.js": "./lib/adapters/xhr.js"
    };
    var bugs = {
    	url: "https://github.com/axios/axios/issues"
    };
    var bundleDependencies = false;
    var bundlesize = [
    	{
    		path: "./dist/axios.min.js",
    		threshold: "5kB"
    	}
    ];
    var dependencies = {
    	"follow-redirects": "1.5.10",
    	"is-buffer": "^2.0.2"
    };
    var deprecated = false;
    var description = "Promise based HTTP client for the browser and node.js";
    var devDependencies = {
    	bundlesize: "^0.5.7",
    	coveralls: "^2.11.9",
    	"es6-promise": "^4.0.5",
    	grunt: "^1.0.1",
    	"grunt-banner": "^0.6.0",
    	"grunt-cli": "^1.2.0",
    	"grunt-contrib-clean": "^1.0.0",
    	"grunt-contrib-nodeunit": "^1.0.0",
    	"grunt-contrib-watch": "^1.0.0",
    	"grunt-eslint": "^19.0.0",
    	"grunt-karma": "^2.0.0",
    	"grunt-ts": "^6.0.0-beta.3",
    	"grunt-webpack": "^1.0.18",
    	"istanbul-instrumenter-loader": "^1.0.0",
    	"jasmine-core": "^2.4.1",
    	karma: "^1.3.0",
    	"karma-chrome-launcher": "^2.0.0",
    	"karma-coverage": "^1.0.0",
    	"karma-firefox-launcher": "^1.0.0",
    	"karma-jasmine": "^1.0.2",
    	"karma-jasmine-ajax": "^0.1.13",
    	"karma-opera-launcher": "^1.0.0",
    	"karma-safari-launcher": "^1.0.0",
    	"karma-sauce-launcher": "^1.1.0",
    	"karma-sinon": "^1.0.5",
    	"karma-sourcemap-loader": "^0.3.7",
    	"karma-webpack": "^1.7.0",
    	"load-grunt-tasks": "^3.5.2",
    	minimist: "^1.2.0",
    	sinon: "^1.17.4",
    	typescript: "^2.0.3",
    	"url-search-params": "^0.6.1",
    	webpack: "^1.13.1",
    	"webpack-dev-server": "^1.14.1"
    };
    var homepage = "https://github.com/axios/axios";
    var keywords = [
    	"xhr",
    	"http",
    	"ajax",
    	"promise",
    	"node"
    ];
    var license = "MIT";
    var main = "index.js";
    var name = "axios";
    var repository = {
    	type: "git",
    	url: "git+https://github.com/axios/axios.git"
    };
    var scripts = {
    	build: "NODE_ENV=production grunt build",
    	coveralls: "cat coverage/lcov.info | ./node_modules/coveralls/bin/coveralls.js",
    	examples: "node ./examples/server.js",
    	postversion: "git push && git push --tags",
    	preversion: "npm test",
    	start: "node ./sandbox/server.js",
    	test: "grunt test && bundlesize",
    	version: "npm run build && grunt version && git add -A dist && git add CHANGELOG.md bower.json package.json"
    };
    var typings = "./index.d.ts";
    var version = "0.18.1";
    var _package = {
    	_from: _from,
    	_id: _id,
    	_inBundle: _inBundle,
    	_integrity: _integrity,
    	_location: _location,
    	_phantomChildren: _phantomChildren,
    	_requested: _requested,
    	_requiredBy: _requiredBy,
    	_resolved: _resolved,
    	_shasum: _shasum,
    	_spec: _spec,
    	_where: _where,
    	author: author,
    	browser: browser$1,
    	bugs: bugs,
    	bundleDependencies: bundleDependencies,
    	bundlesize: bundlesize,
    	dependencies: dependencies,
    	deprecated: deprecated,
    	description: description,
    	devDependencies: devDependencies,
    	homepage: homepage,
    	keywords: keywords,
    	license: license,
    	main: main,
    	name: name,
    	repository: repository,
    	scripts: scripts,
    	typings: typings,
    	version: version
    };

    var _package$1 = /*#__PURE__*/Object.freeze({
        _from: _from,
        _id: _id,
        _inBundle: _inBundle,
        _integrity: _integrity,
        _location: _location,
        _phantomChildren: _phantomChildren,
        _requested: _requested,
        _requiredBy: _requiredBy,
        _resolved: _resolved,
        _shasum: _shasum,
        _spec: _spec,
        _where: _where,
        author: author,
        browser: browser$1,
        bugs: bugs,
        bundleDependencies: bundleDependencies,
        bundlesize: bundlesize,
        dependencies: dependencies,
        deprecated: deprecated,
        description: description,
        devDependencies: devDependencies,
        homepage: homepage,
        keywords: keywords,
        license: license,
        main: main,
        name: name,
        repository: repository,
        scripts: scripts,
        typings: typings,
        version: version,
        'default': _package
    });

    var pkg = getCjsExportFromNamespace(_package$1);

    var httpFollow = followRedirects.http;
    var httpsFollow = followRedirects.https;






    /*eslint consistent-return:0*/
    var http_1 = function httpAdapter(config) {
      return new Promise(function dispatchHttpRequest(resolve, reject) {
        var data = config.data;
        var headers = config.headers;
        var timer;

        // Set User-Agent (required by some servers)
        // Only set header if it hasn't been set in config
        // See https://github.com/axios/axios/issues/69
        if (!headers['User-Agent'] && !headers['user-agent']) {
          headers['User-Agent'] = 'axios/' + pkg.version;
        }

        if (data && !utils.isStream(data)) {
          if (Buffer.isBuffer(data)) ; else if (utils.isArrayBuffer(data)) {
            data = new Buffer(new Uint8Array(data));
          } else if (utils.isString(data)) {
            data = new Buffer(data, 'utf-8');
          } else {
            return reject(createError(
              'Data after transformation must be a string, an ArrayBuffer, a Buffer, or a Stream',
              config
            ));
          }

          // Add Content-Length header if data exists
          headers['Content-Length'] = data.length;
        }

        // HTTP basic authentication
        var auth = undefined;
        if (config.auth) {
          var username = config.auth.username || '';
          var password = config.auth.password || '';
          auth = username + ':' + password;
        }

        // Parse url
        var parsed = url.parse(config.url);
        var protocol = parsed.protocol || 'http:';

        if (!auth && parsed.auth) {
          var urlAuth = parsed.auth.split(':');
          var urlUsername = urlAuth[0] || '';
          var urlPassword = urlAuth[1] || '';
          auth = urlUsername + ':' + urlPassword;
        }

        if (auth) {
          delete headers.Authorization;
        }

        var isHttps = protocol === 'https:';
        var agent = isHttps ? config.httpsAgent : config.httpAgent;

        var options = {
          path: buildURL$1(parsed.path, config.params, config.paramsSerializer).replace(/^\?/, ''),
          method: config.method,
          headers: headers,
          agent: agent,
          auth: auth
        };

        if (config.socketPath) {
          options.socketPath = config.socketPath;
        } else {
          options.hostname = parsed.hostname;
          options.port = parsed.port;
        }

        var proxy = config.proxy;
        if (!proxy && proxy !== false) {
          var proxyEnv = protocol.slice(0, -1) + '_proxy';
          var proxyUrl = process.env[proxyEnv] || process.env[proxyEnv.toUpperCase()];
          if (proxyUrl) {
            var parsedProxyUrl = url.parse(proxyUrl);
            proxy = {
              host: parsedProxyUrl.hostname,
              port: parsedProxyUrl.port
            };

            if (parsedProxyUrl.auth) {
              var proxyUrlAuth = parsedProxyUrl.auth.split(':');
              proxy.auth = {
                username: proxyUrlAuth[0],
                password: proxyUrlAuth[1]
              };
            }
          }
        }

        if (proxy) {
          options.hostname = proxy.host;
          options.host = proxy.host;
          options.headers.host = parsed.hostname + (parsed.port ? ':' + parsed.port : '');
          options.port = proxy.port;
          options.path = protocol + '//' + parsed.hostname + (parsed.port ? ':' + parsed.port : '') + options.path;

          // Basic proxy authorization
          if (proxy.auth) {
            var base64 = new Buffer(proxy.auth.username + ':' + proxy.auth.password, 'utf8').toString('base64');
            options.headers['Proxy-Authorization'] = 'Basic ' + base64;
          }
        }

        var transport;
        if (config.transport) {
          transport = config.transport;
        } else if (config.maxRedirects === 0) {
          transport = isHttps ? https : http;
        } else {
          if (config.maxRedirects) {
            options.maxRedirects = config.maxRedirects;
          }
          transport = isHttps ? httpsFollow : httpFollow;
        }

        if (config.maxContentLength && config.maxContentLength > -1) {
          options.maxBodyLength = config.maxContentLength;
        }

        // Create the request
        var req = transport.request(options, function handleResponse(res) {
          if (req.aborted) return;

          // Response has been received so kill timer that handles request timeout
          clearTimeout(timer);
          timer = null;

          // uncompress the response body transparently if required
          var stream = res;
          switch (res.headers['content-encoding']) {
          /*eslint default-case:0*/
          case 'gzip':
          case 'compress':
          case 'deflate':
            // add the unzipper to the body stream processing pipeline
            stream = stream.pipe(zlib.createUnzip());

            // remove the content-encoding in order to not confuse downstream operations
            delete res.headers['content-encoding'];
            break;
          }

          // return the last request in case of redirects
          var lastRequest = res.req || req;

          var response = {
            status: res.statusCode,
            statusText: res.statusMessage,
            headers: res.headers,
            config: config,
            request: lastRequest
          };

          if (config.responseType === 'stream') {
            response.data = stream;
            settle(resolve, reject, response);
          } else {
            var responseBuffer = [];
            stream.on('data', function handleStreamData(chunk) {
              responseBuffer.push(chunk);

              // make sure the content length is not over the maxContentLength if specified
              if (config.maxContentLength > -1 && Buffer.concat(responseBuffer).length > config.maxContentLength) {
                stream.destroy();
                reject(createError('maxContentLength size of ' + config.maxContentLength + ' exceeded',
                  config, null, lastRequest));
              }
            });

            stream.on('error', function handleStreamError(err) {
              if (req.aborted) return;
              reject(enhanceError(err, config, null, lastRequest));
            });

            stream.on('end', function handleStreamEnd() {
              var responseData = Buffer.concat(responseBuffer);
              if (config.responseType !== 'arraybuffer') {
                responseData = responseData.toString('utf8');
              }

              response.data = responseData;
              settle(resolve, reject, response);
            });
          }
        });

        // Handle errors
        req.on('error', function handleRequestError(err) {
          if (req.aborted) return;
          reject(enhanceError(err, config, null, req));
        });

        // Handle request timeout
        if (config.timeout && !timer) {
          timer = setTimeout(function handleRequestTimeout() {
            req.abort();
            reject(createError('timeout of ' + config.timeout + 'ms exceeded', config, 'ECONNABORTED', req));
          }, config.timeout);
        }

        if (config.cancelToken) {
          // Handle cancellation
          config.cancelToken.promise.then(function onCanceled(cancel) {
            if (req.aborted) return;

            req.abort();
            reject(cancel);
          });
        }

        // Send the request
        if (utils.isStream(data)) {
          data.pipe(req);
        } else {
          req.end(data);
        }
      });
    };

    var DEFAULT_CONTENT_TYPE = {
      'Content-Type': 'application/x-www-form-urlencoded'
    };

    function setContentTypeIfUnset(headers, value) {
      if (!utils.isUndefined(headers) && utils.isUndefined(headers['Content-Type'])) {
        headers['Content-Type'] = value;
      }
    }

    function getDefaultAdapter() {
      var adapter;
      if (typeof XMLHttpRequest !== 'undefined') {
        // For browsers use XHR adapter
        adapter = xhr;
      } else if (typeof process !== 'undefined') {
        // For node use HTTP adapter
        adapter = http_1;
      }
      return adapter;
    }

    var defaults = {
      adapter: getDefaultAdapter(),

      transformRequest: [function transformRequest(data, headers) {
        normalizeHeaderName(headers, 'Content-Type');
        if (utils.isFormData(data) ||
          utils.isArrayBuffer(data) ||
          utils.isBuffer(data) ||
          utils.isStream(data) ||
          utils.isFile(data) ||
          utils.isBlob(data)
        ) {
          return data;
        }
        if (utils.isArrayBufferView(data)) {
          return data.buffer;
        }
        if (utils.isURLSearchParams(data)) {
          setContentTypeIfUnset(headers, 'application/x-www-form-urlencoded;charset=utf-8');
          return data.toString();
        }
        if (utils.isObject(data)) {
          setContentTypeIfUnset(headers, 'application/json;charset=utf-8');
          return JSON.stringify(data);
        }
        return data;
      }],

      transformResponse: [function transformResponse(data) {
        /*eslint no-param-reassign:0*/
        if (typeof data === 'string') {
          try {
            data = JSON.parse(data);
          } catch (e) { /* Ignore */ }
        }
        return data;
      }],

      /**
       * A timeout in milliseconds to abort a request. If set to 0 (default) a
       * timeout is not created.
       */
      timeout: 0,

      xsrfCookieName: 'XSRF-TOKEN',
      xsrfHeaderName: 'X-XSRF-TOKEN',

      maxContentLength: -1,

      validateStatus: function validateStatus(status) {
        return status >= 200 && status < 300;
      }
    };

    defaults.headers = {
      common: {
        'Accept': 'application/json, text/plain, */*'
      }
    };

    utils.forEach(['delete', 'get', 'head'], function forEachMethodNoData(method) {
      defaults.headers[method] = {};
    });

    utils.forEach(['post', 'put', 'patch'], function forEachMethodWithData(method) {
      defaults.headers[method] = utils.merge(DEFAULT_CONTENT_TYPE);
    });

    var defaults_1 = defaults;

    function InterceptorManager() {
      this.handlers = [];
    }

    /**
     * Add a new interceptor to the stack
     *
     * @param {Function} fulfilled The function to handle `then` for a `Promise`
     * @param {Function} rejected The function to handle `reject` for a `Promise`
     *
     * @return {Number} An ID used to remove interceptor later
     */
    InterceptorManager.prototype.use = function use(fulfilled, rejected) {
      this.handlers.push({
        fulfilled: fulfilled,
        rejected: rejected
      });
      return this.handlers.length - 1;
    };

    /**
     * Remove an interceptor from the stack
     *
     * @param {Number} id The ID that was returned by `use`
     */
    InterceptorManager.prototype.eject = function eject(id) {
      if (this.handlers[id]) {
        this.handlers[id] = null;
      }
    };

    /**
     * Iterate over all the registered interceptors
     *
     * This method is particularly useful for skipping over any
     * interceptors that may have become `null` calling `eject`.
     *
     * @param {Function} fn The function to call for each interceptor
     */
    InterceptorManager.prototype.forEach = function forEach(fn) {
      utils.forEach(this.handlers, function forEachHandler(h) {
        if (h !== null) {
          fn(h);
        }
      });
    };

    var InterceptorManager_1 = InterceptorManager;

    /**
     * Transform the data for a request or a response
     *
     * @param {Object|String} data The data to be transformed
     * @param {Array} headers The headers for the request or response
     * @param {Array|Function} fns A single function or Array of functions
     * @returns {*} The resulting transformed data
     */
    var transformData = function transformData(data, headers, fns) {
      /*eslint no-param-reassign:0*/
      utils.forEach(fns, function transform(fn) {
        data = fn(data, headers);
      });

      return data;
    };

    var isCancel = function isCancel(value) {
      return !!(value && value.__CANCEL__);
    };

    /**
     * Determines whether the specified URL is absolute
     *
     * @param {string} url The URL to test
     * @returns {boolean} True if the specified URL is absolute, otherwise false
     */
    var isAbsoluteURL = function isAbsoluteURL(url) {
      // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
      // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
      // by any combination of letters, digits, plus, period, or hyphen.
      return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
    };

    /**
     * Creates a new URL by combining the specified URLs
     *
     * @param {string} baseURL The base URL
     * @param {string} relativeURL The relative URL
     * @returns {string} The combined URL
     */
    var combineURLs = function combineURLs(baseURL, relativeURL) {
      return relativeURL
        ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '')
        : baseURL;
    };

    /**
     * Throws a `Cancel` if cancellation has been requested.
     */
    function throwIfCancellationRequested(config) {
      if (config.cancelToken) {
        config.cancelToken.throwIfRequested();
      }
    }

    /**
     * Dispatch a request to the server using the configured adapter.
     *
     * @param {object} config The config that is to be used for the request
     * @returns {Promise} The Promise to be fulfilled
     */
    var dispatchRequest = function dispatchRequest(config) {
      throwIfCancellationRequested(config);

      // Support baseURL config
      if (config.baseURL && !isAbsoluteURL(config.url)) {
        config.url = combineURLs(config.baseURL, config.url);
      }

      // Ensure headers exist
      config.headers = config.headers || {};

      // Transform request data
      config.data = transformData(
        config.data,
        config.headers,
        config.transformRequest
      );

      // Flatten headers
      config.headers = utils.merge(
        config.headers.common || {},
        config.headers[config.method] || {},
        config.headers || {}
      );

      utils.forEach(
        ['delete', 'get', 'head', 'post', 'put', 'patch', 'common'],
        function cleanHeaderConfig(method) {
          delete config.headers[method];
        }
      );

      var adapter = config.adapter || defaults_1.adapter;

      return adapter(config).then(function onAdapterResolution(response) {
        throwIfCancellationRequested(config);

        // Transform response data
        response.data = transformData(
          response.data,
          response.headers,
          config.transformResponse
        );

        return response;
      }, function onAdapterRejection(reason) {
        if (!isCancel(reason)) {
          throwIfCancellationRequested(config);

          // Transform response data
          if (reason && reason.response) {
            reason.response.data = transformData(
              reason.response.data,
              reason.response.headers,
              config.transformResponse
            );
          }
        }

        return Promise.reject(reason);
      });
    };

    /**
     * Create a new instance of Axios
     *
     * @param {Object} instanceConfig The default config for the instance
     */
    function Axios(instanceConfig) {
      this.defaults = instanceConfig;
      this.interceptors = {
        request: new InterceptorManager_1(),
        response: new InterceptorManager_1()
      };
    }

    /**
     * Dispatch a request
     *
     * @param {Object} config The config specific for this request (merged with this.defaults)
     */
    Axios.prototype.request = function request(config) {
      /*eslint no-param-reassign:0*/
      // Allow for axios('example/url'[, config]) a la fetch API
      if (typeof config === 'string') {
        config = utils.merge({
          url: arguments[0]
        }, arguments[1]);
      }

      config = utils.merge(defaults_1, {method: 'get'}, this.defaults, config);
      config.method = config.method.toLowerCase();

      // Hook up interceptors middleware
      var chain = [dispatchRequest, undefined];
      var promise = Promise.resolve(config);

      this.interceptors.request.forEach(function unshiftRequestInterceptors(interceptor) {
        chain.unshift(interceptor.fulfilled, interceptor.rejected);
      });

      this.interceptors.response.forEach(function pushResponseInterceptors(interceptor) {
        chain.push(interceptor.fulfilled, interceptor.rejected);
      });

      while (chain.length) {
        promise = promise.then(chain.shift(), chain.shift());
      }

      return promise;
    };

    // Provide aliases for supported request methods
    utils.forEach(['delete', 'get', 'head', 'options'], function forEachMethodNoData(method) {
      /*eslint func-names:0*/
      Axios.prototype[method] = function(url, config) {
        return this.request(utils.merge(config || {}, {
          method: method,
          url: url
        }));
      };
    });

    utils.forEach(['post', 'put', 'patch'], function forEachMethodWithData(method) {
      /*eslint func-names:0*/
      Axios.prototype[method] = function(url, data, config) {
        return this.request(utils.merge(config || {}, {
          method: method,
          url: url,
          data: data
        }));
      };
    });

    var Axios_1 = Axios;

    /**
     * A `Cancel` is an object that is thrown when an operation is canceled.
     *
     * @class
     * @param {string=} message The message.
     */
    function Cancel(message) {
      this.message = message;
    }

    Cancel.prototype.toString = function toString() {
      return 'Cancel' + (this.message ? ': ' + this.message : '');
    };

    Cancel.prototype.__CANCEL__ = true;

    var Cancel_1 = Cancel;

    /**
     * A `CancelToken` is an object that can be used to request cancellation of an operation.
     *
     * @class
     * @param {Function} executor The executor function.
     */
    function CancelToken(executor) {
      if (typeof executor !== 'function') {
        throw new TypeError('executor must be a function.');
      }

      var resolvePromise;
      this.promise = new Promise(function promiseExecutor(resolve) {
        resolvePromise = resolve;
      });

      var token = this;
      executor(function cancel(message) {
        if (token.reason) {
          // Cancellation has already been requested
          return;
        }

        token.reason = new Cancel_1(message);
        resolvePromise(token.reason);
      });
    }

    /**
     * Throws a `Cancel` if cancellation has been requested.
     */
    CancelToken.prototype.throwIfRequested = function throwIfRequested() {
      if (this.reason) {
        throw this.reason;
      }
    };

    /**
     * Returns an object that contains a new `CancelToken` and a function that, when called,
     * cancels the `CancelToken`.
     */
    CancelToken.source = function source() {
      var cancel;
      var token = new CancelToken(function executor(c) {
        cancel = c;
      });
      return {
        token: token,
        cancel: cancel
      };
    };

    var CancelToken_1 = CancelToken;

    /**
     * Syntactic sugar for invoking a function and expanding an array for arguments.
     *
     * Common use case would be to use `Function.prototype.apply`.
     *
     *  ```js
     *  function f(x, y, z) {}
     *  var args = [1, 2, 3];
     *  f.apply(null, args);
     *  ```
     *
     * With `spread` this example can be re-written.
     *
     *  ```js
     *  spread(function(x, y, z) {})([1, 2, 3]);
     *  ```
     *
     * @param {Function} callback
     * @returns {Function}
     */
    var spread = function spread(callback) {
      return function wrap(arr) {
        return callback.apply(null, arr);
      };
    };

    /**
     * Create an instance of Axios
     *
     * @param {Object} defaultConfig The default config for the instance
     * @return {Axios} A new instance of Axios
     */
    function createInstance(defaultConfig) {
      var context = new Axios_1(defaultConfig);
      var instance = bind(Axios_1.prototype.request, context);

      // Copy axios.prototype to instance
      utils.extend(instance, Axios_1.prototype, context);

      // Copy context to instance
      utils.extend(instance, context);

      return instance;
    }

    // Create the default instance to be exported
    var axios = createInstance(defaults_1);

    // Expose Axios class to allow class inheritance
    axios.Axios = Axios_1;

    // Factory for creating new instances
    axios.create = function create(instanceConfig) {
      return createInstance(utils.merge(defaults_1, instanceConfig));
    };

    // Expose Cancel & CancelToken
    axios.Cancel = Cancel_1;
    axios.CancelToken = CancelToken_1;
    axios.isCancel = isCancel;

    // Expose all/spread
    axios.all = function all(promises) {
      return Promise.all(promises);
    };
    axios.spread = spread;

    var axios_1 = axios;

    // Allow use of default import syntax in TypeScript
    var default_1 = axios;
    axios_1.default = default_1;

    var axios$1 = axios_1;

    /**
     * 接口代理模块
     * @author wdzxc
     */

    function bind$1(fn, thisArg) {
        return function wrap() {
            let args = new Array(arguments.length);
            for (let i = 0; i < args.length; i++) {
                args[i] = arguments[i];
            }
            return fn.apply(thisArg, args);
        };
    }

    function ExtendAxios() {
        return bind$1(function (options) {
            let callback = options.success;
            let errCallback = options.error;
            // method & type
            options.method = options.method || options.type;
            // responseType & dataType
            options.responseType = options.responseType || options.dataType;
            // headers
            options.headers = options.headers || {};
            if (options.responseType === 'html') {
                options.headers["Accept"] = "text/html, */*";
            }
            if (options.responseType === 'text') {
                options.headers["Accept"] = "text/plain, */*";
            }
            // options polyfill
            if (options.method === 'post' || options.method === 'put' || options.method === 'patch') {
                options.data = options.data || options.params;
                // if (JSON.stringify(axiosOptions.params) === JSON.stringify(axiosOptions.data)) {
                //     delete axiosOptions.params;
                // }
                options.headers["Content-Type"] = options.headers["Content-Type"]
                    || "application/json";
            }
            if (options.beforeSend) {
                let fakeXhr = {
                    setRequestHeader(key, value) {
                        options.headers[key] = value;
                    }
                };
                options.beforeSend(fakeXhr);
            }
            axios$1(options).then((response) => {
                callback(response.data, response.status);
            }).catch((error) => {
                errCallback(error);
            });
        }, axios$1);
    }

    const extendAxios = new ExtendAxios();
    const PROXY_METHOD_CONFIG_MARK = "__proxy_method_config_mark__";

    //=========================================================
    //  参数说明 & 默认参数
    //=========================================================
    let defaultOptions$1 = {
        //----------------------------------
        //  接口数据默认配置
        //----------------------------------
        // 默认域名
        origin: null,
        // xhr send 前钩子函数
        beforeSend: new Function,
        // 参数过滤 - 默认过滤掉null字段
        paramsFilter(params) {
            Object.keys(params).forEach(key => {
                if (params[key] === null) delete params[key];
            });
            return params;
        },
        // 成功回调的数据预处理
        dataFilter: null,
        // 默认失败回调
        callback: new Function,
        errCallback: new Function,
        // ajax default
        type: "get",
        dataType: "json",
        contentType: "application/json; charset=utf-8",
        traditional: true,
        jsonp: null,        // jsonpCallback
        // modify header
        fakeHeaders: null,
    };

    /**
     * 接口配置项
     * @typedef {Object}    OriginAjaxConfig
     * // 基础配置
     * @property {string}   [pathname=]             - 接口路径 路径可以是相对路径也可以是绝对路径，按书写方式区分
     * @property {string}   [url=]                  - url 通常是绝对路径
     * @property {string}   [origin=""]             - 接口域名 如果一个接口存在origin配置，则会和pathname进行拼接作为具体路径。
     * @property {string}   [type="GET"]            - 操作方法 GET|POST
     * @property {string}   [dataType="json"]       - 数据类型 json|jsonp
     * @property {string}   [jsonp="jsonpCallback"] - jsonp 回调函数名
     * @property {string}   [paramsType=""]         - 参数类型
     * @property {function} [paramsFilter=""]       - 参数预处理
     * @property {string}   [dataPath=""]           - 数据路径
     * @property {function} [dataFilter=""]         - 数据预处理
     * @property {object}   [pathParams=]           - 路径参数
     * @property {boolean}  [merge=false]           - 合并
     * @property {boolean}  [queue=false]           - 排队
     * @property {boolean}  [single=false]          - 单一最后有效
     * @property {number}   [validTime=0]           - 有效时间，单位为秒，可以为浮点数
     * @property {*}        [params=null]           - 接口查询参数，{} [] "" ， 如果没有接口调用时没有对应参数，会使用这里配置的默认配置进行补充
     * @property {string}   [name=""]               - 描述接口名称，参考使用
     * @property {*}        [returns=null]          - 接口成功状态返回数据的参考格式，只是针对数据部分。例如: 返回数据:{ success:true,data:{} } => returns : {} （为data对象）
     * // 高级配置
     * @property {function} [errCallback=]          - 默认失败回调函数
     * @property {function} [ajax=]                 - ajax 调用
     * @property {function} [beforeSend=]           - 发送前钩子函数，可以用于修改header等
     * @property {function} [model=]                - 接口模型
     * @property {function} [successHandler=]       - 返回数据成功状态判断过滤器
     *
     * @example
     * {
     *      getGoodsList: {
     *           name: "GoodsList",
     *           pathname: "/goods/findAll",
     *           params: {type:1},
     *           returns: [{
     *              "id": "001",
     *              "name": "Iphone4"
     *           }]
     *      }
     *      // ...
     * }
     */

    // 接口请求装配工具箱
    const toolkit = {

        // 生成接口最终配置
        getInterfaceConfig(key, proxyObj) {
            // 接口配置列表
            const configList = objClone(proxyObj.$configMap);

            // 预处理optionsList
            function doMixin(config) {
                let result = config;
                if (config.mixins) {
                    let mixinTarget = config.mixins;
                    if (isString(mixinTarget) || isObject(mixinTarget)) {
                        mixinTarget = [mixinTarget];
                    }
                    if (isArray(mixinTarget)) {
                        mixinTarget.reverse().map(mixin => {
                            if (isString(mixin)) {
                                return configList[mixin];
                            }
                            if (isObject(mixin)) return mixin;
                            // console.error("InterfaceProxy Mixin ERROR.");
                        }).forEach(mixin => {
                            if (mixin.mixins) {
                                // 循环mixin处理
                                result = objCover(doMixin(mixin), result);
                            } else {
                                result = objCover(objClone(mixin), objClone(result));
                            }
                        });
                    }
                }
                return result;
            }

            // 拼装配置列表
            let unitOptions = doMixin(configList[key]);
            let troopOptions = proxyObj.$additionalOptions;
            let finalConfig = objCover(objCover(objClone(defaultOptions$1), troopOptions), unitOptions);
            finalConfig.$unitOptions = unitOptions;
            finalConfig.$troopOptions = troopOptions;
            finalConfig.$defaultOptions = defaultOptions$1;
            return finalConfig;
        },
        // 生成Ajax最终配置
        getAjaxConfig(args, interfaceConfig, baseConfig) {
            // 基础AJAX配置
            let ajaxConfig = objCover(objClone(interfaceConfig), baseConfig);
            /*----------------------------------------------------
             支持参数传入格式
             > callback
             > callback,errCallback
             > params,callback,errCallback
             > [pArg1,pArg2...],callback,errCallback
             > 依照Res配置默认构造方法填充未传入参数值
             ----------------------------------------------------*/
            let defaultParams = ajaxConfig.params;
            // 捕获参数
            let captureParams = {};
            let captureCallback = null;
            let captureErrCallback = null;
            let _Symbol = null;
            if (typeof Symbol !== 'undefined') {
                _Symbol = Symbol;
            }
            // 默认参数值 由接口配置生成 如果默认参数配置的类型为函数，则获取函数执行结果
            Object.keys(defaultParams || {}).forEach(key => {
                let value = defaultParams[key];
                if (value === Number
                    || value === String
                    || value === Array
                    || value === Boolean
                    || value === Function
                    || value === _Symbol
                    || value === Object) {
                    captureParams[key] = null;
                    return;
                }
                if (isObject(value)) {
                    value = value.default || null;
                }
                if (isFunction(value)) {
                    captureParams[key] = value();
                    return;
                }
                captureParams[key] = value;
            });
            if (baseConfig) {
                if (isObject(interfaceConfig.params)) {
                    objCover(captureParams, ajaxConfig.params);
                }
                if (isObject(interfaceConfig.data)) {
                    ajaxConfig.data = objCover(interfaceConfig.data, ajaxConfig.data);
                }
            } else {
                // 根据args捕获参数
                // 分析函数传入参数列表，提取 params,callback,errCallback
                let multiParamPattern = true;
                let multiParamNum = 0;
                [].forEach.call(args, function (param) {
                    // 提取成功与失败回调函数
                    if (isFunction(param)) {
                        if (!captureCallback) {
                            captureCallback = param;
                            return;
                        }
                        if (!captureErrCallback) {
                            captureErrCallback = param;
                            return;
                        }
                    }
                    // 提取参数列表  单独 {} 配置
                    if (isObject(param)) {
                        objCover(captureParams, param);
                        multiParamPattern = false;
                    }
                    // 多参数配置
                    if (multiParamPattern && !isFunction(param)) {
                        let attrNum = 0;
                        for (let an in defaultParams) {
                            if (!defaultParams.hasOwnProperty(an)) continue;
                            if (attrNum++ === multiParamNum) {
                                captureParams[an] = param;
                                multiParamNum++;
                                break;
                            }
                        }
                    }
                });
            }
            // URL处理基本参数处理
            let url = (function () {
                if (ajaxConfig.url) return ajaxConfig.url;
                if (ajaxConfig.origin) {
                    return ajaxConfig.origin + ajaxConfig.pathname;
                }
                else return ajaxConfig.pathname;
            })(ajaxConfig);
            // url path重构 pathParams
            let pathParams = ajaxConfig.pathParams;
            if (pathParams) {
                Object.keys(pathParams).forEach(key => {
                    let value = pathParams[key];
                    url = url.replace(new RegExp(`:${key}`), value);
                });
            }

            // ! 参数过滤器 -------------------------------------------------
            if (isFunction(ajaxConfig.paramsFilter)) {
                captureParams = ajaxConfig.paramsFilter(captureParams);
            }
            // --------------------------------------------------------------

            ajaxConfig.method = ajaxConfig.method || ajaxConfig.type;
            ajaxConfig.url = url;
            ajaxConfig.data = ajaxConfig.data || captureParams;
            if(!~['post','put','patch'].indexOf(ajaxConfig.method.toLowerCase())){
                ajaxConfig.params = captureParams;
            }
            ajaxConfig.callback = captureCallback;
            ajaxConfig.errCallback = captureErrCallback;
            // 生成请求标记
            if (ajaxConfig.validTime > 0) {
                ajaxConfig.requestMark = this.generateDataMark(ajaxConfig);
            }
            return ajaxConfig;
        },
        /**
         * 生成接口数据交换操作函数
         * @this module:InterfaceProxy
         * @param key 单个接口配置
         * @param proxyObj 接口代理对象配置
         * @returns {Function}
         * @ignore
         */
        buildInterfaceMethod: function (key, proxyObj) {
            /**
             * 构造接口调用函数
             * @param {*=} params        - 查询参数
             * @param {*=} callback      - 成功回调
             * @param {*=} errCallback   - 失败回调
             * @param {*=} methodConfig  - 代理方法配置
             */
            let proxyMethod = function (params, callback, errCallback, methodConfig) {
                let args = [].slice.call(arguments);
                // 根据各级配置生成最终配置
                let interfaceConfig = toolkit.getInterfaceConfig(key, proxyObj);
                let baseConfig = null;

                args = args.filter(function (arg) {
                    if (isObject(arg) && arg[PROXY_METHOD_CONFIG_MARK]) {
                        baseConfig = arg;
                        return false;
                    }
                    return true;
                });

                // 根据参数和接口配置生成ajaxConfig
                let ajaxConfig = toolkit.getAjaxConfig(args, interfaceConfig, baseConfig);

                // 重新包装callback
                let wrapCallback = function () {
                    /**
                     * 默认失败回调
                     * 1. 如果没有配置通用回调和专用回调，则使用默认回调
                     * 2. 如果配置了通用回调，没有专用回调，则使用通用回调
                     * 3. 如果配置了通用回调和专用回调，则先调用专用回调，专用回调返回true则继续调用通用回调
                     * 4. 如果配置了通用回调和专用回调，则先调用专用回调，专用回调返回false，则过程结束
                     * 专用回调[cb,then] -> 通用回调[unitOptions,troopOptions,defaultOptions]
                     */
                    // 在存在配置回调 和 默认回调的情况下，包装新的回调函数
                    if (ajaxConfig.errCallback && interfaceConfig.errCallback && (ajaxConfig.errCallback !== interfaceConfig.errCallback)) {
                        ajaxConfig.errCallback = (function (callback, defaultCallback) {
                            return function () {
                                return callback.apply(this, arguments) && defaultCallback.apply(this, arguments);
                            };
                        })(ajaxConfig.errCallback, interfaceConfig.errCallback);
                    }
                    // 无参数回调 则先使用默认回调作为ajax回调
                    if (!ajaxConfig.errCallback) {
                        ajaxConfig.errCallback = interfaceConfig.errCallback;
                    }
                    /**
                     * 默认成功回调
                     */
                    if (!ajaxConfig.callback) {
                        ajaxConfig.callback = interfaceConfig.callback;
                    }
                };

                // Promise形式封装
                let agent = {};

                // .then
                agent.then = function (callback, errCallback) {
                    if (callback) {
                        let _captureCallback = ajaxConfig.callback;
                        ajaxConfig.callback = function () {
                            _captureCallback && _captureCallback.apply(this, arguments);
                            callback.apply(this, arguments);
                        };
                    }
                    if (errCallback) {
                        // ajax回调 = 默认回调 则 参数回调不存在
                        // ajax回调替换为 链式回调
                        if (ajaxConfig.errCallback === interfaceConfig.errCallback) {
                            ajaxConfig.errCallback = errCallback;
                        } else {
                            // 参数回调 默认回调 都存在则先执行参数回调再执行链式回调
                            let _captureErrCallback = ajaxConfig.errCallback;
                            ajaxConfig.errCallback = function () {
                                return (_captureErrCallback && _captureErrCallback.apply(this, arguments))
                                    || errCallback.apply(this, arguments);
                            };
                            wrapCallback();
                        }
                    }
                    return agent;
                };

                // .catch .error
                agent.catch = agent.error = function (errCallback) {
                    // catch方法是 .then(null,rejection)的包装
                    agent.then(null, errCallback);
                    return agent;
                };

                // .finally
                agent.finally = function (callback) {
                    let _captureCallback = ajaxConfig.callback;
                    let _captureErrCallback = ajaxConfig.errCallback;
                    ajaxConfig.callback = function () {
                        _captureCallback && _captureCallback.apply(this, arguments);
                        callback.apply(this, arguments);
                    };
                    ajaxConfig.errCallback = function () {
                        _captureErrCallback && _captureErrCallback.apply(this, arguments);
                        callback.apply(this, arguments);
                    };
                    ajaxConfig.finallyCallback = callback;
                    return {};
                };

                // errCallback
                wrapCallback();

                // .loadingSwitch
                agent.loadingSwitch = function (target, key) {
                    let _target = target, _key = key;
                    interfaceConfig.optLoadingSwitch = function (value) {
                        _target[_key] = value;
                    };
                    return agent;
                };

                // do ajax => async start
                setTimeout(function () {
                    toolkit.doAjax(interfaceConfig, ajaxConfig);
                }, 1);

                return agent;
            };
            // wrap axios methods with base config
            ['delete', 'get', 'head', 'options'].forEach(function (method) {
                proxyMethod[method] = function (params, config) {
                    let methodConfig = config || {};
                    methodConfig[PROXY_METHOD_CONFIG_MARK] = true;
                    methodConfig['method'] = method;
                    methodConfig['params'] = objCover(methodConfig['params'] || {}, params || {});
                    return proxyMethod(methodConfig);
                };
            });
            ['post', 'put', 'patch'].forEach(function (method) {
                proxyMethod[method] = function (data, config) {
                    let methodConfig = config || {};
                    methodConfig[PROXY_METHOD_CONFIG_MARK] = true;
                    methodConfig['method'] = method;
                    methodConfig['data'] = data;
                    return proxyMethod(methodConfig);
                };
            });
            return proxyMethod;
        },
        // 发起请求
        doAjax(interfaceConfig, ajaxConfig) {
            // Ajax 开始前 interceptor
            if (this.beforeAjax(interfaceConfig, ajaxConfig)) return;
            // 请求响应统一处理
            let onResponse = (type, data) => {
                // Ajax 结束后 interceptor
                if (this.afterAjax(interfaceConfig, ajaxConfig)) return;
                let resMap = {
                    success: data => {
                        this.AjaxGetSuccessProcess({
                            data: data.data,
                            status: data.status,
                        }, interfaceConfig, ajaxConfig);
                    },
                    localSuccess: data => {
                        this.AjaxGetSuccessProcess({
                            data: data.data,
                            status: data.status,
                            isLocalObtain: true
                        }, interfaceConfig, ajaxConfig);
                    },
                    error: error => {
                        this.AjaxGetFailProcess(error, interfaceConfig, ajaxConfig);
                    }
                };
                resMap[type] && resMap[type](data);
            };
            // 绑定请求处理函数
            ajaxConfig.success = function (data, status) {
                onResponse('success', {
                    data: data,
                    status: status
                });
            };
            ajaxConfig.error = function (status) {
                onResponse('error', status);
            };
            // ------------------------------ 开始请求数据 ----------------------------
            // 尝试搜索本地数据
            if (interfaceConfig.validTime) {
                let localData = this.getAlreadyObtain(ajaxConfig.requestMark);
                if (localData) {
                    // 匹配到本地数据
                    onResponse('localSuccess', {
                        data: localData,
                        status: 200
                    });
                    return;
                }
            }
            // ajax 获取数据
            if (ajaxConfig.dataType === 'jsonp') {
                ajaxJsonp(objClone(ajaxConfig));
                return;
            }
            (interfaceConfig.ajax || extendAxios)(objClone(ajaxConfig));
        },
        // 请求前
        beforeAjax(interfaceConfig, ajaxConfig) {
            let flag = this.requestManager.start(interfaceConfig, ajaxConfig);
            if (!flag) {
                // 加载中开关
                if (isFunction(interfaceConfig.optLoadingSwitch)) {
                    interfaceConfig.optLoadingSwitch(true);
                }
            }
            return flag;
        },
        // 请求后
        afterAjax(interfaceConfig, ajaxConfig) {
            // 加载中开关
            if (isFunction(interfaceConfig.optLoadingSwitch)) {
                interfaceConfig.optLoadingSwitch(false);
            }
            return this.requestManager.finish(interfaceConfig, ajaxConfig);
        },
        /**
         * Ajax数据预处理
         * 200+>接收数据处理:成功
         * @param response
         * @param {} response.data
         * @param interfaceConfig
         * @param ajaxConfig
         * @this module:InterfaceProxy
         * @ignore
         */
        AjaxGetSuccessProcess: function (response, interfaceConfig, ajaxConfig) {
            // 基础配置
            let opt = objCover({}, response);

            // 配置项转化
            let data = opt.data;
            let status = opt.status;
            let callback = ajaxConfig.callback;
            let errCallback = ajaxConfig.errCallback;
            let isLocalObtain = opt.isLocalObtain;

            // 数据缓存处理
            if (!isLocalObtain && interfaceConfig.validTime) {
                this.repository.push({
                    key: ajaxConfig.requestMark,
                    timeStamp: +new Date(),
                    data: data,
                    validTime: interfaceConfig.validTime
                });
            }

            // callback
            let callbackEx = null;
            if (callback) {
                callbackEx = function (extractData) {
                    // 提取数据
                    arguments[0] = interfaceConfig.dataPath ? getObjAttr(extractData, interfaceConfig.dataPath) : arguments[0];
                    arguments[0] = isFunction(interfaceConfig.dataFilter) ? interfaceConfig.dataFilter(arguments[0], interfaceConfig.dataType) : arguments[0];
                    return callback.apply(null, arguments);
                };
            }

            /*----------------------------------------------------
             接口返回值类型（200）
             > 空 : 视为成功
             > {success:true,...}
             判断后进行成功或失败回调
             > data | msg  : 成功时尝试提取data，失败时尝试提取msg
             ----------------------------------------------------*/

            // 自定义成功状态判断机制
            if (isFunction(interfaceConfig.successHandler)) {
                let result = interfaceConfig.successHandler(data) || {};
                if (result.success === true) {
                    return callbackEx && callbackEx(result.data);
                }
                if (result.success === false) {
                    return errCallback(result.msg, data);
                }
            }
            // 当返回数据为空 > 直接当做成功
            if (!data) {
                return callbackEx && callbackEx(null);
            }
            // 依据success标识提取返回数据
            if (data.success === true) {
                // 提取data数据
                if (!isUndefined(data.data)) {
                    return callbackEx && callbackEx(data.data, data, status);
                }
                return callbackEx && callbackEx(data, data, status);
            }
            if (data.success === false) {
                return errCallback && errCallback(data.msg, data);
            }
            // 当返回数据不包含success字段 > 视为成功
            if (isUndefined(data.success)) {
                return callbackEx && callbackEx(data, data);
            }
        },
        /**
         * 200X>接收数据处理:失败
         * @param error
         * @param interfaceConfig
         * @param ajaxConfig
         * @this module:InterfaceProxy
         * @ignore
         */
        AjaxGetFailProcess: function (error, interfaceConfig, ajaxConfig) {
            // AJAX 失败
            ajaxConfig.errCallback && ajaxConfig.errCallback(error.message, error);
        },
        /**
         * 生成数据标记
         * @param ajaxConfig
         * @returns {string}
         * @this module:InterfaceProxy
         * @ignore
         */
        generateDataMark: function (ajaxConfig) {
            let dataMark = JSON.stringify(ajaxConfig);
            dataMark = sha1.hex_hmac(dataMark, '888');
            return dataMark;
        },
        /**
         * 获得数据
         * @param key
         * @returns {*}
         * @this module:InterfaceProxy
         * @ignore
         */
        getAlreadyObtain: function (key) {
            let data = this.repository.get(key);
            if (data) {
                return data.responseData;
            }
            return null;
        },
        // 请求队列
        requestManager: (function () {
            function RequestManager() {
                // 写点啥呢...
            }

            RequestManager.prototype = [];
            // 这个骚操作拖慢了数组性能...
            // RequestManager.prototype.constructor = RequestManager;
            RequestManager.prototype.requestQueryEqual = function (a, b) {
                return this.interfaceConfigEqual(a.interfaceConfig, b.interfaceConfig) && objEqual(a.ajaxConfig.data, b.ajaxConfig.data);
            };
            RequestManager.prototype.interfaceConfigEqual = function (a, b) {
                return JSON.stringify(a) === JSON.stringify(b);
            };
            RequestManager.prototype.requestEqual = function (a, b) {
                return this.interfaceConfigEqual(a.interfaceConfig, b.interfaceConfig) && a.ajaxConfig.data === b.ajaxConfig.data;
            };
            RequestManager.prototype.removeItem = function (interfaceConfig, ajaxConfig) {
                let removed = null;
                this.find((r, index) => {
                    if (interfaceConfig && ajaxConfig) {
                        if (this.requestEqual(r, {
                            interfaceConfig: interfaceConfig,
                            ajaxConfig: ajaxConfig
                        })) {
                            removed = r;
                            this.splice(index, 1);
                            return true;
                        }
                    } else if (interfaceConfig && this.interfaceConfigEqual(interfaceConfig, interfaceConfig)) {
                        removed = r;
                        this.splice(index, 1);
                        return true;
                    }
                });
                return removed;
            };
            RequestManager.prototype.start = function (interfaceConfig, ajaxConfig) {
                // 唯一
                if (interfaceConfig.single) {
                    this.removeItem(interfaceConfig);
                }
                // 合并
                if (interfaceConfig.merge) {
                    const sameRequest = this.find(r => this.requestQueryEqual(r, {
                        interfaceConfig: interfaceConfig,
                        ajaxConfig: ajaxConfig
                    }));
                    if (sameRequest) {
                        // 异步执行包装回调的代码 Reason : .then .error .finally
                        const callback = sameRequest.ajaxConfig.callback,
                            errCallback = sameRequest.ajaxConfig.errCallback;
                        if (callback) {
                            sameRequest.ajaxConfig.callback = function () {
                                callback.apply(this, arguments);
                                ajaxConfig.callback.apply(this, arguments);
                            };
                        }
                        if (errCallback) {
                            sameRequest.ajaxConfig.errCallback = function () {
                                errCallback.apply(this, arguments);
                                ajaxConfig.errCallback.apply(this, arguments);
                            };
                        }
                        return true;
                    }
                }
                // 入队
                this.push.call(this, {
                    interfaceConfig: interfaceConfig,
                    ajaxConfig: ajaxConfig,
                    timeStamp: new Date()
                });
            };
            RequestManager.prototype.finish = function (interfaceConfig, ajaxConfig) {
                // 排队
                if (interfaceConfig.queue) {
                    // 找到同接口存在的前一个请求
                    let r = null;
                    this.find(request => {
                        if (this.requestEqual(request, {interfaceConfig: interfaceConfig, ajaxConfig: ajaxConfig})) {
                            return false;
                        }
                        if (this.interfaceConfigEqual(request.interfaceConfig, interfaceConfig) && request.ajaxConfig !== ajaxConfig) {
                            r = request;
                            return true;
                        }
                    });
                    if (r) {
                        const callback = r.ajaxConfig.callback, errCallback = r.ajaxConfig.errCallback;
                        const nextCallback = ajaxConfig.callback, nextErrCallback = ajaxConfig.errCallback;
                        // 将回调附加到next函数上，等待队列前的请求调用next进行回调触发
                        let next;
                        ajaxConfig.callback = function () {
                            const arg = objClone(arguments);
                            next = function () {
                                nextCallback.apply(this, arg);
                            };
                        };
                        ajaxConfig.errCallback = function () {
                            const arg = objClone(arguments);
                            next = function () {
                                nextErrCallback.apply(this, arg);
                            };
                        };
                        if (callback) {
                            r.ajaxConfig.callback = function () {
                                callback.apply(this, arguments);
                                next && next();
                            };
                        }
                        if (errCallback) {
                            r.ajaxConfig.errCallback = function () {
                                errCallback.apply(this, arguments);
                                next && next();
                            };
                        }
                    }
                }
                // 清除
                if (!this.removeItem(interfaceConfig, ajaxConfig)) {
                    // single : 如果请求已从队列中移除，则终止处理流程
                    if (interfaceConfig.single) return true;
                }
            };
            return new RequestManager();
        })(),
        // 持久化存储
        repository: (function () {
            if(typeof window === 'undefined'){
                global.window = {};
            }
            let ls = window.localStorage;
            return {
                push: function (params) {
                    // 保存新的数据
                    ls[params.key] = JSON.stringify({
                        timeStamp: params.timeStamp,
                        validTime: params.validTime,
                        responseData: params.data
                    });
                },
                get: function (key) {
                    this.clean();
                    return JSON.parse(ls.getItem(key) || 'null');
                },
                // 清理过期缓存
                clean: function () {
                    // 清除过期缓存
                    for (let i = 0; i < ls.length; i++) {
                        // 42db81629a 3582822d6b b6b83bbe1d 2378e6aeeb
                        if (/\w{40}/.test(ls.key(i))) {
                            let storeData = JSON.parse(ls.getItem(ls.key(i)) || 'null');
                            if (storeData) {
                                if ((new Date().getTime() - storeData.timeStamp) / 1000 > storeData.validTime) {
                                    ls.removeItem(ls.key(i));
                                }
                            }
                        }
                    }
                }
            };
        })()
    };


    /**
     * 接口管理器
     * new InterfaceProxy(interfaceConfig,additionalOptions) 来生成接口操作对象(return)
     * @module InterfaceProxy
     * @param {object}              configMap 接口描述配置信息unitInfo[]
     * @param {OriginAjaxConfig|Array}    additionalOptions 接口基础配置信息
     * @constructor
     */
    function InterfaceProxy(configMap, additionalOptions) {
        // 基础配置信息
        this.$additionalOptions = (isArray(additionalOptions) ? additionalOptions : [additionalOptions || {}]).reduce((obj1, obj2) => {
            return objCover(objClone(obj1), obj2);
        }, {});
        // 接口配置JSON
        this.$configMap = configMap;
        // 装配工具箱
        this.$toolkit = toolkit;
        // model
        this.$model = this.$additionalOptions.model || null;
        // 初始化接口 - 自动生成所有接口操作方法
        this.__init__();
    }

    InterfaceProxy.prototype = /**@lends module:InterfaceProxy# */{
        /**
         * 初始化
         * @this module:InterfaceProxy
         * @ignore
         */
        __init__: function () {
            // 变量交换
            let configList = this.$configMap;
            // 提供 iProxy[name] 操作方法 | 将 构造的getData在调用时的this指向getData[pn]本体
            let iProxy = this;
            // 动态生成接口代理对象操作方法
            for (let pn in configList) {
                if (!configList.hasOwnProperty(pn)) continue;
                /**
                 * @typedef {function} interfaceMethod          - 自动生成的接口操作方法
                 * @property {OriginAjaxConfig} interfaceInfo   - 接口信息，来源接口配置
                 * @property {OriginAjaxConfig} info            - 接口信息，来源接口配置
                 * @property {string} url                       - 接口URL
                 */
                iProxy[pn] = toolkit.buildInterfaceMethod(pn, this);
                iProxy[pn].$options = iProxy[pn].info = configList[pn];
                iProxy[pn].url = iProxy[pn].info.url || (iProxy[pn].info.origin || '') + iProxy[pn].info.pathname;
            }
        }
    };

    // 导出Hashes
    InterfaceProxy.Hashes = sha1;
    // 导出Ajax
    // InterfaceProxy.Ajax = Ajax;
    InterfaceProxy.Ajax = extendAxios;
    InterfaceProxy.axios = axios$1;
    // 修改全局默认配置API
    InterfaceProxy.setDefault = options => {
        objCover(defaultOptions$1, options);
    };

    /**
     * 入口文件
     * @author wdzxc
     */

    exports.$wd = $wd;
    exports.FileUploader = FileUploader;
    exports.InterfaceProxy = InterfaceProxy;
    exports.axios = axios$1;
    exports.default = $wd;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
